/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : BrowserDbgData.cpp
 * DATE   : 06.02.2007
 *-------------------------------------------------------------------
 * Okno w ktorym uzytkownik wpisuje nazwy, ktorych wartosci chce 
 * odczytac z debuggera. Wywolanie odczytu wartosci zmiennych mozna
 * spowodowac na dwa sposoby: uzytkonik wpisal nowa zmienna,
 * skonczylo sie odczytywanie wartosci zmiennych w aktualnym zakresie.
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "BrowserDbgData.h"
#include "EventsController.h"
#include <QKeyEvent>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const qint32      BrowserDbgData::NAME_COLUMN  = 0;
const qint32      BrowserDbgData::VALUE_COLUMN = 1;
const char* const BrowserDbgData::NO_SYMBOL    = QT_TR_NOOP( "No symbol \"%1\" in current context" );


//*******************************************************************
// BrowserDbgData                                        CONSTRUCTOR
//*******************************************************************
BrowserDbgData::BrowserDbgData( QWidget* const in_parent )
: DbgDataTable ( in_parent )
, d_status     ( None )
{
	EventsController::instance()->append( this, Event::DBG_ERROR );
	EventsController::instance()->append( this, Event::DBG_ITEM_TYPE );
	EventsController::instance()->append( this, Event::DBG_ITEM_VALUE );
	EventsController::instance()->append( this, Event::DBG_AFTER_SCOPE );
	EventsController::instance()->append( this, Event::DBG_FINISHED );
	
	connect( this, SIGNAL( cellChanged(int,int) ), this, SLOT( cell_changed(int,int) ) );
	add_row();
}
// end of BrowserDbgData

//*******************************************************************
// ~BrowserDbgData                                        DESTRUCTOR
//*******************************************************************
BrowserDbgData::~BrowserDbgData()
{
	EventsController::instance()->remove( this );
}
// end of ~BrowserDbgData

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void BrowserDbgData::customEvent( QEvent* const in_event )
{
	Event* const event = dynamic_cast< Event* >( in_event );
	const qint32 type = static_cast< qint32 >( event->type() );

	switch( type ) {
		case Event::DBG_ERROR:
			error();
			break;
		case Event::DBG_ITEM_TYPE:
			item_type( event );
			break;
		case Event::DBG_ITEM_VALUE:
			item_value( event );
			break;
		case Event::DBG_AFTER_SCOPE:
			get_prv_data();
			d_idx = -1;
			d_status = Type;
			next_type();
			break;
		case Event::DBG_FINISHED:
			finished();
			add_row();
			break;
	}
}
// end of customEvent

//*******************************************************************
// keyPressEvent                                   PRIVATE inherited
//*******************************************************************
void BrowserDbgData::keyPressEvent( QKeyEvent* in_event )
{
	QTableWidget::keyPressEvent( in_event );

	if( None == d_status ) {
		switch( in_event->key() ) {
			case Qt::Key_Insert:
				insert_row();
				break;
			case Qt::Key_Delete:
				remove_row();
				break;
		}
	}
}
// end of keyPressEvent

//*******************************************************************
// insert_row                                                PRIVATE
//*******************************************************************
void BrowserDbgData::insert_row()
{
	QString name;
	get_item( rowCount() - 1, NAME_COLUMN, name );
	if( !name.isEmpty() ) {
		add_row();
	}
}
// end of insert_row

//*******************************************************************
// remove_row                                                PRIVATE
//*******************************************************************
void BrowserDbgData::remove_row()
{
	d_disabled_slot = true;
	removeRow( currentRow() );
	d_disabled_slot = false;
}
// end of remove_row

//*******************************************************************
// edit                                            PRIVATE inherited
//-------------------------------------------------------------------
// Uruchomiecie pola edycyjnego w celu wprowadzenia nazwy zmiennej.
// Mozemy sie na to zgodzic jesli:
//   - nie trwa proces odczytywania wartosci zmiennych,
//   - jesli dotyczy to kolumny numer 0 (kolumna nazw zmiennych).
//*******************************************************************
bool BrowserDbgData::edit( const QModelIndex& in_index, EditTrigger in_trigger, QEvent* const in_event )
{
	bool retval = false;
	if( ( None == d_status ) &&  ( NAME_COLUMN == currentColumn() ) ) {
		retval = QTableWidget::edit( in_index, in_trigger, in_event );
	}
	return retval;
}
// end of edit

//*******************************************************************
// error                                                     PRIVATE
//-------------------------------------------------------------------
// Jesli wystapil blad przy odczycie wartosci aktualnej zmiennej
// oznacza to, ze zmienna nie jest widziana w aktualnym zakresie.
// Wpisujemy stosowny komunikat zamiast wartosci zmiennej i
// kontynuujemy odczyt wartosci dla kolejnych zmiennych.
//*******************************************************************
void BrowserDbgData::error()
{
	if( d_status ) {
		QString name;
		get_item( d_idx, NAME_COLUMN, name );
		set_item( d_idx, VALUE_COLUMN, tr( NO_SYMBOL ).arg( name ) );

		switch( d_status ) {
			case Type:
				next_type();
				break;
			case Value:
				next_value();
				break;
		}
	}
}
// end of error

//*******************************************************************
// next_type                                                 PRIVATE
//*******************************************************************
void BrowserDbgData::next_type()
{
	if( Type == d_status ) {
		bool ok = false;
		
		if( ++d_idx < rowCount() ) {
			QString name;
			if( get_item( d_idx, NAME_COLUMN, name ) ) {
				EventsController::instance()->send_event( Event::DBG_GET_ITEM_TYPE, name );
				ok = true;
			}
		}
		
		if( !ok ) {
			d_idx = -1;
			d_status = Value;
			next_value();
		}
	}
}
// end of next_type

//*******************************************************************
// item_type                                                 PRIVATE
//*******************************************************************
void BrowserDbgData::item_type( Event* const in_event )
{
	if( Type == d_status ) {
		if( d_idx != -1 ) {
			set_tool_tip( d_idx, NAME_COLUMN, in_event->d_data.at( 0 ).toString() );
			next_type();
		}
	}
}
// end of item_type

//*******************************************************************
// next_value                                                PRIVATE
//*******************************************************************
void BrowserDbgData::next_value()
{
	if( Value == d_status ) {
		bool ok = false;

		if( ++d_idx < rowCount() ) {
			QString name, type;
			if( get_item( d_idx, NAME_COLUMN, name ) && get_tool_tip( d_idx, NAME_COLUMN, type ) ) {
				EventsController::instance()->send_event( Event::DBG_GET_ITEM_VALUE, name, type );
				ok = true;
			}
		}

		if( !ok ) {
			resize_columns();
			d_prv_data.clear();
			d_status = None;
			d_idx = -1;

			EventsController::instance()->send_event( Event::DBG_AFTER_DATA );
		}
	}
}
// end of send_next

//*******************************************************************
// item_value                                                PRIVATE
//*******************************************************************
void BrowserDbgData::item_value( Event* const in_event )
{
	if( Value == d_status ) {
		set_item( d_idx, VALUE_COLUMN, in_event->d_data.at( 0 ).toString() );
		resize_columns();
		colors( d_idx );
		next_value();
	}
}
// end of data_item

//*******************************************************************
// cell_changed                                         PRIVATE slot
//-------------------------------------------------------------------
// Po zmianie zawartosci komorki, czyli po wpisaniu zmiennej
// inicjujemy odczyt wartosci wszystkich zmiennych.
//*******************************************************************
void BrowserDbgData::cell_changed( const int in_row, const int in_col )
{
	if( !d_disabled_slot && ( None == d_status ) ) {
		if( ( NAME_COLUMN == in_col ) && ( rowCount() > 0 ) ) {
			QTableWidgetItem* const itm = item( in_row, NAME_COLUMN );
			if( itm->text().isEmpty() ) {
				if( in_row < ( rowCount() - 1 ) ) {
					removeRow( in_row );
				}
				else  {
					set_tool_tip( in_row, NAME_COLUMN, "?" );
					set_item( in_row, VALUE_COLUMN, "?" );
				}
			}
			else {
				get_prv_data();
				d_idx = -1;
				d_status = Type;
				next_type();
			}
		}
	}
}
// end of cell_chaned
