/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : BrowserDbgScope.cpp
 * DATE   : 06.02.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "BrowserDbgScope.h"
#include "EventsController.h"
#include <QHeaderView>
#include <QStringList>
#include <QHeaderView>
#include <QtDebug>
#include <string>
#include <iostream>

/*------- local constants:
-------------------------------------------------------------------*/
const qint32  BrowserDbgScope::NAME_COLUMN         = 0;
const qint32  BrowserDbgScope::VALUE_COLUMN        = 1;
const QString BrowserDbgScope::UNKNOWN_MARK        = "?";


//*******************************************************************
// BrowserDbgScope                                       CONSTRUCTOR
//*******************************************************************
BrowserDbgScope::BrowserDbgScope( QWidget* const in_parent )
: DbgDataTable ( in_parent )
, d_status     ( None )
{
   setEditTriggers( NoEditTriggers );

   EventsController::instance()->append( this, Event::DBG_ERROR );
   EventsController::instance()->append( this, Event::DBG_SCOPE );
   EventsController::instance()->append( this, Event::DBG_ITEM_TYPE );
   EventsController::instance()->append( this, Event::DBG_ITEM_VALUE );
   EventsController::instance()->append( this, Event::DBG_FINISHED );
}
// end of BrowserDbgScope

//*******************************************************************
// ~BrowserDbgScope                                       DESTRUCTOR
//*******************************************************************
BrowserDbgScope::~BrowserDbgScope()
{
   EventsController::instance()->remove( this );
}
// end of ~BrowserDbgScope

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void BrowserDbgScope::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast< qint32 >( event->type() );

   switch( type ) {
      case Event::DBG_ERROR:
         error();
         break;
      case Event::DBG_SCOPE:
         scope( event );
         break;
      case Event::DBG_ITEM_TYPE:
         item_type( event );
         break;
      case Event::DBG_ITEM_VALUE:
         item_value( event );
         break;
      case Event::DBG_FINISHED:
         finished();
         break;
   }
}
// end of customEvent

//*******************************************************************
// error                                                     PRIVATE
//*******************************************************************
void BrowserDbgScope::error()
{
   if( d_idx != -1 ) {
      removeRow( d_idx );

      switch( d_status ) {
         case Type:
            next_type();
            break;
         case Value:
            next_value();
            break;
      }
   }
}
// end of error

//*******************************************************************
// scope                                                     PRIVATE
//-------------------------------------------------------------------
// Zostaly przyslane informacje o NAZWACH wszystkich zmiennych
// bedacych w zasiegu. Po wpisaniu ich do tabeli w kolejnych
// wierszach w kolumnie 0, inicjujemy odczyt ich typow i wartosci.
//*******************************************************************
void BrowserDbgScope::scope( Event* const in_event )
{
   get_prv_data();
   
   const QStringList data = in_event->d_data.at( 0 ).toStringList();
   qint32 row = 0;

   QStringList::const_iterator it = data.begin();
   const QStringList::const_iterator end = data.end();
   while( it != end ) {
      set_data( row++, *it );
      ++it;
   }

   remove_rows( row );

   d_idx = -1;
   d_status = Type;
   next_type();
}
// end of scope

//*******************************************************************
// next_type                                                 PRIVATE
//*******************************************************************
void BrowserDbgScope::next_type()
{
   if( Type == d_status ) {
      bool ok = false;

      if( ++d_idx < rowCount() ) {
         QString name;
         if( get_item( d_idx, NAME_COLUMN, name ) ) {
            EventsController::instance()->send_event( Event::DBG_GET_ITEM_TYPE, name );
            ok = true;
         }
      }

      if( !ok ) {
         d_idx = -1;
         d_status = Value;
         next_value();
      }
   }
}
// end of next_type

//*******************************************************************
// item_type                                                 PRIVATE
//*******************************************************************
void BrowserDbgScope::item_type( Event* const in_event )
{
   if( Type == d_status ) {
      if( d_idx != -1 ) {
         set_tool_tip( d_idx, NAME_COLUMN, in_event->d_data.at( 0 ).toString() );
         next_type();
      }
   }
}
// end of item_type

//*******************************************************************
// next_value                                                PRIVATE
//*******************************************************************
void BrowserDbgScope::next_value()
{
   if( Value == d_status ) {
      bool ok = false;
      
      if( ++d_idx < rowCount() ) {
         QString name, type;
         if( get_item( d_idx, NAME_COLUMN, name ) && get_tool_tip( d_idx, NAME_COLUMN, type ) ) {
            EventsController::instance()->send_event( Event::DBG_GET_ITEM_VALUE, name, type );
            ok = true;
         }
      }

      if( !ok ) {
         resize_columns();
         d_prv_data.clear();
         d_status = None;
         d_idx = -1;

         EventsController::instance()->send_event( Event::DBG_AFTER_SCOPE );
      }
   }
}
// end of send_next

//*******************************************************************
// item_value                                                PRIVATE
//*******************************************************************
void BrowserDbgScope::item_value( Event* const in_event )
{
   if( Value == d_status ) {
      if( d_idx != -1 ) {
         set_item( d_idx, VALUE_COLUMN, in_event->d_data.at( 0 ).toString() );
         colors( d_idx );
         next_value();
      }
   }
}
// end of item_value

//*******************************************************************
// set_data                                                  PRIVATE
//*******************************************************************
void BrowserDbgScope::set_data( const qint32 in_idx, const QString& in_name )
{
   if( !in_name.isEmpty() ) {
      const qint32 row = ( in_idx < rowCount() ) ? in_idx : add_row( in_idx );

      set_item( row, NAME_COLUMN, in_name );
      set_tool_tip( row, NAME_COLUMN, UNKNOWN_MARK );
      set_item( row, VALUE_COLUMN, UNKNOWN_MARK );
   }
}
// end of set_data
