/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ClasserParser.h
 * DATE   : 8.10.2006
 *******************************************************************/
#ifndef INCLUDED_ClassParser_h
#define INCLUDED_ClassParser_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_Shared_h
#include "Shared.h"
#endif // INCLUDED_Shared_h

#ifndef INCLUDED_QThread
#define INCLUDED_QThread
#include <QThread>
#endif // INCLUDED_QThread

#ifndef INCLUDED_QString
#define INCLUDED_QString
#include <QString>
#endif // INCLUDED_QString

#ifndef INCLUDED_QVector
#define INCLUDED_QVector
#include <QVector>
#endif // INCLUDED_QVector

#ifndef INCLUDED_QStack
#define INCLUDED_QStack
#include <QStack>
#endif // INCLUDED_QStack

#ifndef INCLUDED_QHash
#define INCLUDED_QHash
#include <QHash>
#endif // INCLUDED_QHash


/*------- class declaration:
-------------------------------------------------------------------*/
class ParsedItem
{
public:
   enum {
      NO_SCOPE  = 0x00000,
      PRIVATE   = 0x00001,
      PROTECTED = 0x00010,
      PUBLIC    = 0x00100,
      SLOT      = 0x01000,
      SIGNAL    = 0x10000,
   };
   typedef enum {
      GLOBAL = 0,
      CLASS,
      STRUCT,
      UNION,
      ENUM,
      MEMBER,
      METHOD,
   } Type;

   typedef enum {
      UNKNOWN = -1,
      DECL,
      IMPL,
      DECL_IMPL,
      TYPEDEF,
   } What;

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   ParsedItem();
   virtual ~ParsedItem();
   ParsedItem( const ParsedItem& );
   ParsedItem& operator=( const ParsedItem& );
   bool operator==( const ParsedItem& in_other );

// ******* METHODS *******
private:
   void            reset ();
   QString scope_as_text( quint32 in_scope ) const;
public:
   void            print () const;
   const QString&   file () const { return d_file;   }
   const QString&   name () const { return d_name;   }
   Type             type () const { return d_type;   }
   What             what () const { return d_what;   }
   const QString&   text () const { return d_text;   }
   unsigned int     scope () const { return d_scope;  }
   const QString& parent () const { return d_parent; }
   qint32            pos () const { return d_pos;    }
   //..............................................   
   void   file ( const QString& in_file   ) { d_file   = in_file  ; d_file.squeeze();   }
   void   name ( const QString& in_name   ) { d_name   = in_name  ; d_name.squeeze();   }
   void   type ( const Type     in_type   ) { d_type   = in_type  ;                     }
   void   what ( const What     in_what   ) { d_what   = in_what  ;                     }
   void   text ( const QString& in_text   ) { d_text   = in_text  ; d_text.squeeze();   }
   void  scope ( const quint32& in_scope  ) { d_scope  = in_scope ;                     }
   void parent ( const QString& in_parent ) { d_parent = in_parent; d_parent.squeeze(); }
   void    pos ( const qint32   in_pos    ) { d_pos    = in_pos   ;                     }
//******* MEMBER *******
private:
   QString d_file;
   QString d_name;
   Type    d_type;
   What    d_what;
   QString d_text;
   quint32 d_scope;
   QString d_parent;
   qint32  d_pos;
};


class ClassParser : public QThread
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   ClassParser();
private:
   ClassParser( const ClassParser& );
   ClassParser& operator=( const ClassParser& );

// ******* TYPES *******
public:
   typedef QHash< QString, QVector< ParsedItem > > ClassItems;
   
// ******* MEMBERS *******
private:
   QString         d_buffer;
   QString         d_fpath;
   qint32          d_thread_idx;
   bool            d_fread;
   ClassItems      d_class_items;
   QStack<QString> d_class;
   qint32          d_idx;
   qint32          d_len;
   quint32         d_scope;

// ******* METHODS *******
public:
   const QString&        path () { return d_fpath; }
   void              set_data ( const QString& in_fpath, qint32 in_thread_idx );
   void              set_data ( const QString& in_fpath, qint32 in_thread_idx, const QString& in_text );
   const ClassItems&     data () const { return d_class_items; }
private:
   void                 print ();
   void                 reset ();
   void                   run ();
   bool             read_file ( const QString& in_fpath );
   qint32               parse ();
   void          omit_comment ( qint32& out_pos );
   void            parse_item ( qint32& in_pos );
   void           parse_block ( qint32& in_pos );
   void             end_block ( qint32& in_pos );
   void           parse_scope ( qint32& in_pos );
   void            parse_func ( qint32& in_pos );
   bool         search_end_of ( const QChar& in_smark, const QChar& in_emark );
   qint32            first_of ( const QString& in_set );
   void             skip_char ();
   void           skip_string ();
   void      skip_cpp_comment ();
   void        skip_c_comment ();
   void      skip_preprocesor ();
   void         go_after_char ( const QChar& in_char );
   bool         is_acceptable () const;
   bool         is_acceptable ( qint32 in_idx ) const;
   bool        is_cpp_comment () const;
   bool   is_end_of_c_comment () const;
   bool is_begin_of_c_comment () const;
   bool        index_is_valid () const;
private slots:
   void         finished_slot ();
signals:
   void              finished ( qint32 );
};

#endif // INCLUDED_ClassParser_h
