/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ClassesView.cpp
 * DATE   : 3.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ClassesView.h"
#include "Classer.h"
#include "Config.h"
#include "Shared.h"
#include "EventsController.h"
#include <QHeaderView>
#include <QStandardItemModel>
#include <QToolTip>
#include <QtDebug>

//*******************************************************************
// ClassesView                                           CONSTRUCTOR
//*******************************************************************
ClassesView::ClassesView( QWidget* const in_parent )
: QTreeWidget   ( in_parent )
, d_root_item   ( 0 )
, d_org_palette ( palette() )
, d_org_font    ( font() )
{
   header()->hide();
   setItemsExpandable( true );
   setRootIsDecorated( true );
   setEditTriggers( QTreeView::NoEditTriggers );
   setColumnCount( 1 );
   setHorizontalScrollMode( ScrollPerPixel );
   
   set_properties();
   
   EventsController::instance()->append( this, Event::PROJECT_OPENED );
   EventsController::instance()->append( this, Event::PROJECT_CLOSED );
   EventsController::instance()->append( this, Event::PROJECT_PARSED );
   EventsController::instance()->append( this, Event::PROJECT_REPARSED );
   EventsController::instance()->append( this, Event::CLTREE_PROP_CHANGED );
      
   connect( this, SIGNAL( itemDoubleClicked( QTreeWidgetItem*, int )), this, SLOT( double_clicked( QTreeWidgetItem* )));
}
// end of ClassesView

//*******************************************************************
// ~ClassesView                                           DESTRUCTOR
//*******************************************************************
ClassesView::~ClassesView()
{
   EventsController::instance()->remove( this );
}
// end of ~ClassesView

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void ClassesView::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast< qint32 >( event->type() );
   
   switch( type ) {
      case Event::PROJECT_OPENED:
         opened( event );
         break;
      case Event::PROJECT_CLOSED:
         closed( event );
         break;
      case Event::PROJECT_PARSED:
         project_parsed();
         break;
      case Event::PROJECT_REPARSED:
         project_reparsed();
         break;
      case Event::CLTREE_PROP_CHANGED:
         set_properties();
         break;
   }
}
// end of customEvent

//*******************************************************************
// double_clicked                                       PRIVATE slot
//*******************************************************************
void ClassesView::double_clicked( QTreeWidgetItem* const in_item )
{
   if( in_item ) {
      switch( in_item->data( 0, TYPE_ROLE ).toInt() ) {
         case ParsedItem::CLASS:
            selected_class( in_item );
            break;
         case ParsedItem::MEMBER:
            selected_member( in_item );
            break;
         case ParsedItem::METHOD:
            selected_method( in_item );
            break;
      }
   }
}
// end of double_clicked

//*******************************************************************
// set_properties                                            PRIVATE
//*******************************************************************
void ClassesView::set_properties()
{
   setAlternatingRowColors( Config::instance()->cltree_use_altbkg() );
   
   if( Config::instance()->cltree_use_defaults() ) {
      setPalette( d_org_palette );
      setFont( d_org_font );
   }
   else {
      QPalette p( d_org_palette );
      p.setColor( QPalette::Text, QColor::fromRgba( Config::instance()->cltree_font_color() ) );
      p.setColor( QPalette::Base, QColor::fromRgba( Config::instance()->cltree_base_bkg() ) );
      p.setColor( QPalette::AlternateBase, QColor::fromRgba( Config::instance()->cltree_altbase_bkg() ) );
      setPalette( p );
   
      // Ustawienie nowego fontu.
      QFont fnt;
      fnt.fromString( Config::instance()->cltree_font() );
      setFont( fnt );
   }
}
// end set_properties

//*******************************************************************
// opened                                                    PRIVATE
//*******************************************************************
void ClassesView::opened( Event* const in_event )
{
   if( 0 == d_root_item ) {
      d_root_item = new QTreeWidgetItem( this );
      if( d_root_item ) {
         d_root_item->setText( 0, in_event->d_data.at( 0 ).toString() );
         expandItem( d_root_item );
      }
   }
}
// end of opened

//*******************************************************************
// closed                                                    PRIVATE
//*******************************************************************
void ClassesView::closed( Event* const in_event )
{
   if( d_root_item ) {
      if( d_root_item->text( 0 ) == in_event->d_data.at( 0 ).toString() ) {
         clear();
         d_root_item = 0;
      }
   }
}
// end of closed

//*******************************************************************
// project_parsed                                            PRIVATE
//-------------------------------------------------------------------
// Funkcja wywolywana tylko i wylacznie po otwarciu nowego
// projektu. Okno (i drzewo) zawiera tylko (jedna) pozycje
// oznaczajaca nazwe projektu ('d_root_item').
//*******************************************************************
void ClassesView::project_parsed()
{
   if( d_root_item ) {
      const ClassParser::ClassItems& class_items = Classer::instance()->data();

      const ClassParser::ClassItems::const_iterator end = class_items.end();
      ClassParser::ClassItems::const_iterator it = class_items.begin();

      while( it != end ) {
         const QString class_name = it.key();
         if( !class_name.isEmpty() ) {
            add_class( d_root_item, class_name );
         }
         ++it;
      }
      d_root_item->sortChildren( 0, Qt::AscendingOrder );
   }
}
// end of project_parsed

//*******************************************************************
// add_class                                                 PRIVATE
//-------------------------------------------------------------------
// Dodanie do drzewa klasy i jej wszystkich (!) elementow.
//*******************************************************************
QTreeWidgetItem* ClassesView::add_class( QTreeWidgetItem* const in_parent_item, const QString& in_class_name )
{
   QTreeWidgetItem* retval = 0;
   
   if( in_parent_item ) {
      QTreeWidgetItem* const item = new QTreeWidgetItem( in_parent_item );
      if( item ) {
         item->setText( 0, in_class_name );
         item->setIcon( 0, QIcon( ":/img/class.png" ) );
         item->setData( 0, TYPE_ROLE, ParsedItem::CLASS );
         retval = item;
         
         const QVector< ParsedItem >& items = Classer::instance()->data()[ in_class_name ];

         const QVector< ParsedItem >::const_iterator end = items.end();
         QVector< ParsedItem >::const_iterator it = items.begin();
         
         while( it != end ) {
            if( ( ParsedItem::METHOD == it->type() ) || ( ParsedItem::MEMBER == it->type() ) ) {
               add_item( item, *it );
            }
            ++it;
         }
         sort_childs( item );
      }
   }
   return retval;
}
// end of add_class

//*******************************************************************
// add_item                                                  PRIVATE
//*******************************************************************
void ClassesView::add_item( QTreeWidgetItem* const in_parent_item, const ParsedItem& in_info )
{
   if( in_parent_item ) {
      QTreeWidgetItem* const item = new QTreeWidgetItem( in_parent_item );
      if( item ) {
         setup_item( item, in_info );
      }
   }
}
// end of add_item

//*******************************************************************
// setup_item                                                PRIVATE
//*******************************************************************
void ClassesView::setup_item( QTreeWidgetItem* const in_item, const ParsedItem& in_info )
{
   QString icon_name( ":/img/" );
   if( ParsedItem::MEMBER == in_info.type() ) {
           if( in_info.scope() & ParsedItem::PRIVATE   ) icon_name += "memb_private.png";
      else if( in_info.scope() & ParsedItem::PROTECTED ) icon_name += "memb_protected.png";
      else                                               icon_name += "memb_public.png";
   }
   else {
      if( in_info.scope() & ParsedItem::PRIVATE ) {
         if( in_info.scope() & ParsedItem::SLOT ) icon_name += "slot_private.png";
         else                                     icon_name += "meth_private.png";
      }
      else if( in_info.scope() & ParsedItem::PROTECTED ) {
              if( in_info.scope() & ParsedItem::SLOT   ) icon_name += "slot_protected.png";
         else if( in_info.scope() & ParsedItem::SIGNAL ) icon_name += "signal_protected.png";
         else                                            icon_name += "meth_protected.png";
      }
      else  {
         if( in_info.scope() & ParsedItem::SLOT ) icon_name += "slot_public.png";
          else                                     icon_name += "meth_public.png";
      }
   }

   in_item->setText( 0, in_info.name() );
   in_item->setIcon( 0, QIcon( icon_name ) );
   in_item->setToolTip( 0, in_info.text() );
   
   in_item->setData( 0, TYPE_ROLE , in_info.type() );
   in_item->setData( 0, SCOPE_ROLE, in_info.scope() );
   in_item->setData( 0, FILE_ROLE, in_info.file() );
}
// end of setup_item

//*******************************************************************
// project_reparsed                                          PRIVATE
//*******************************************************************
void ClassesView::project_reparsed()
{
   if( d_root_item ) {
      {   // USUWANIE USUNIETYCH KLAS
         const QStringList& del_classes_list = Classer::instance()->del_classes();
         if( !del_classes_list.empty() ) {
            const QStringList::const_iterator end = del_classes_list.end();
            QStringList::const_iterator it = del_classes_list.begin();
            while( it != end ) {
               QTreeWidgetItem* const item = search_item( ParsedItem::CLASS, *it );
               if( item ) {
                  d_root_item->takeChild( d_root_item->indexOfChild( item ) );
               }
               ++it;
            }
            Classer::instance()->del_classes().clear();
         }
      }

      {   // DODAWANIE NOWYCH KLAS
         const QStringList& new_classes_list = Classer::instance()->new_classes();
         if( !new_classes_list.empty() ) {
            const QStringList::const_iterator end = new_classes_list.end();
            QStringList::const_iterator it = new_classes_list.begin();
            while( it != end ) {
               add_class( d_root_item, *it );
               ++it;
            }
            Classer::instance()->new_classes().clear();
         }
      }
   
      {   // ZMIANY WEWNATRZ KLASY
         const QStringList& mod_classes_list = Classer::instance()->mod_classes();
         if( !mod_classes_list.empty() ) {
            const QStringList::const_iterator end = mod_classes_list.end();
            QStringList::const_iterator it = mod_classes_list.begin();
            while( it != end ) {
               update_class( *it );
               ++it;
            }
            Classer::instance()->mod_classes().clear();
         }
      }

      d_root_item->sortChildren( 0, Qt::AscendingOrder );
   }
}
// end of project_reparsed

//*******************************************************************
// update_class                                              PRIVATE
//-------------------------------------------------------------------
// Uaktualnienie skladnikow klasy wskazanej przez nazwe.
//*******************************************************************
void ClassesView::update_class( const QString& in_class_name )
{
   QTreeWidgetItem* const class_item = search_item( ParsedItem::CLASS, in_class_name );
   if( class_item ) {
      const QVector< ParsedItem >& items = Classer::instance()->data()[ in_class_name ];

      // ETAP 1. DODAWANIE DODANYCH
      const QVector< ParsedItem >::const_iterator end = items.end();
      QVector< ParsedItem >::const_iterator it = items.begin();
      while( it != end ) {
         if( !search_child( class_item, *it ) ) {
//            it->print();
            add_item( class_item, *it );
         }
         ++it;
      }

      // ETAP 2. USUWANIE USUNIETYCH
      for( qint32 i = ( class_item->childCount() - 1 ); i >= 0 ; --i ) {
         QTreeWidgetItem* const item = class_item->child( i );

         const QVector< ParsedItem >::const_iterator end = items.end();
         QVector< ParsedItem >::const_iterator it = items.begin();
         while( it != end ) {
            if( it->type() ==  item->data( 0, TYPE_ROLE ).toInt() ) {
               if( it->scope() == item->data( 0, SCOPE_ROLE ).toUInt() ) {
                  if( it->text() == item->toolTip( 0 ) ) {
                     break;
                  }
               }
            }
            ++it;
         }
         // Poszukiwany element drzewa nie zostal znaleziony w wektorze.
         // Do usuniecia.
         if( it == end ) {
            class_item->takeChild( i );
         }
      }

      // Na koncu sortujemy wszystkie elementy klasy
      sort_childs( class_item );
   }
}
// end of update_class

//*******************************************************************
// selected_class                                            PRIVATE
//*******************************************************************
void ClassesView::selected_class( const QTreeWidgetItem* const in_item )
{
   EventsController::instance()->send_event( Event::CLASS_SELECTED, in_item->text( 0 ) );
}
// end of selected_class

//*******************************************************************
// selected_member                                           PRIVATE
//*******************************************************************
void ClassesView::selected_member( const QTreeWidgetItem* const in_item )
{
   const QTreeWidgetItem* const parent_item = in_item->parent();
   if( parent_item ) {
      if( ParsedItem::CLASS == parent_item->data( 0, TYPE_ROLE ).toInt() ) {
         EventsController::instance()->send_event( Event::MEMBER_SELECTED,
            parent_item->text( 0 ),         // Nazwa klasy.
            in_item->toolTip( 0 ),         // Kompletna deklaracja zmiennej.
            in_item->data( 0, FILE_ROLE )   // W jakim pliku jest deklaracja.
         );
      }
   }
}
// end of selected_member

//*******************************************************************
// selected_method                                           PRIVATE
//*******************************************************************
void ClassesView::selected_method( const QTreeWidgetItem* const in_item )
{
   const QTreeWidgetItem* parent_item = in_item->parent();
   if( parent_item ) {
      if( ParsedItem::CLASS == parent_item->data( 0, TYPE_ROLE ).toInt() ) {
         EventsController::instance()->send_event( Event::METHOD_SELECTED,
            parent_item->text( 0 ),         // Nazwy klasy.
            in_item->toolTip( 0 ),         // Kompletna deklaracja funkcji.
            in_item->data( 0, FILE_ROLE )   // W jakim pliku znajduje sie deklaracja.
         );
      }
   }
}
// end of selected_method

//*******************************************************************
// search_item                                               PRIVATE
//*******************************************************************
QTreeWidgetItem* ClassesView::search_item( const qint32 in_type, const QString& in_name ) const
{
   QTreeWidgetItem* retval = 0;
   
   if( d_root_item ) {
      QModelIndex parent_index = indexFromItem( d_root_item );
      if( parent_index.isValid() ) {
         QModelIndex index = indexBelow( parent_index );
         while( index.isValid() ) {
            QTreeWidgetItem* const item = itemFromIndex( index );
            if( item ) {
               if( ( item->text( 0 ) == in_name ) && ( item->data( 0, TYPE_ROLE ).toInt() == in_type ) ) {
                  retval = item;
                  break;
               }
            }
            index = indexBelow( index );
         }
      }
   }
   return retval;
}
// end of search_item

//*******************************************************************
// search_child                                              PRIVATE
//*******************************************************************
QTreeWidgetItem* ClassesView::search_child( QTreeWidgetItem* in_parent_item, const ParsedItem& in_item ) const
{
   QTreeWidgetItem* retval = 0;
   
   if( in_parent_item ) {
      for( qint32 i = 0; i < in_parent_item->childCount(); ++i ) {
         QTreeWidgetItem* const item = in_parent_item->child( i );
         if( in_item.type() == item->data( 0, TYPE_ROLE ).toInt() ) {
            if( in_item.scope() == item->data( 0, SCOPE_ROLE ).toUInt() ) {
               if( in_item.text() == item->toolTip( 0 ) ) {
                  retval = item;
                  break;
               }
            }
         }
      }
   }
   return retval;
}
// end of search_child


//###################################################################
//#                                                                 #
//#                    S O R T O W A N I E                          #
//#                                                                 #
//###################################################################


//*******************************************************************
// sort_childs                                               PRIVATE
//*******************************************************************
void ClassesView::sort_childs( QTreeWidgetItem* const in_parent_item )
{
   if( in_parent_item ) {
      const qint32 last_idx = in_parent_item->childCount() - 1;
      for( qint32 i = last_idx; i > 0; --i  ) {
         if( cmp_items( in_parent_item->child( i - 1 ), in_parent_item->child( i ) ) < 0 ) {
            swap_items( in_parent_item->child( i - 1 ), in_parent_item->child( i ) );
         }
      }
      for( qint32 i = 1; i <= last_idx; ++i ) {
         qint32 j = i;
         QTreeWidgetItem item = *in_parent_item->child( i );
         while( ( j > 0 ) && ( cmp_items( &item, in_parent_item->child( j - 1 ) ) < 0  ) ) {
            *in_parent_item->child( j ) = *in_parent_item->child( j - 1 );
            --j;
         }
         *in_parent_item->child( j ) = item;
      }
   }
}
// end of sort_positions

//*******************************************************************
// cmp_item                                                  PRIVATE
//*******************************************************************
qint32 ClassesView::cmp_items( QTreeWidgetItem* const in_item1, QTreeWidgetItem* const in_item2 ) const
{
   const QString name1   = in_item1->text( 0 );
   const QString name2   = in_item2->text( 0 );
   
   const qint32 type1    = in_item1->data( 0, TYPE_ROLE ).toInt();
   const qint32 type2    = in_item2->data( 0, TYPE_ROLE ).toInt();
   const quint32 access1 = in_item1->data( 0, SCOPE_ROLE ).toUInt();
   const quint32 access2 = in_item2->data( 0, SCOPE_ROLE ).toUInt();
   
   qint32 retval = 0;
   
   if( type1 < type2 ) {
      retval = -1;
   }
   else if( type1 > type2 ) {
      retval = 1;
   }
   else {
      retval = cmp_access( access1, access2 );
      if( 0 == retval ) {
         retval = cmp_name( in_item1->text( 0 ), in_item2->text( 0 ) );
      }
   }
   
   return retval;
}
// end of cmp_items

//*******************************************************************
// cmp_name                                                  PRIVATE
//*******************************************************************
qint32 ClassesView::cmp_name( const QString& in_name1, const QString& in_name2 ) const
{
   const qint32 len1 = in_name1.length();
   const qint32 len2 = in_name2.length();
   const qint32 len = ( len1 < len2 ) ? len1 : len2;

   qint32 retval = QString::compare( in_name1.mid( 0, len ), in_name2.mid( 0, len ) );
   if( 0 == retval ) {
      retval = QString::compare( in_name1, in_name2 );
   }
   return retval;
}
// end of cmp_name

//*******************************************************************
// cmp_access                                                PRIVATE
//*******************************************************************
qint32 ClassesView::cmp_access( const quint32 in_access1, const quint32 in_access2 ) const
{
   const quint32 access1 = in_access1 & 0x00111;
   const quint32 access2 = in_access2 & 0x00111;
   
   const bool   is_slot1 = ( in_access1 & ParsedItem::SLOT   ) ? true : false;
   const bool   is_slot2 = ( in_access2 & ParsedItem::SLOT   ) ? true : false;
   const bool   is_sig1  = ( in_access1 & ParsedItem::SIGNAL ) ? true : false;
   const bool   is_sig2  = ( in_access2 & ParsedItem::SIGNAL ) ? true : false;

   qint32 retval = 0;

   if( access1 > access2 ) {
      retval = -1;
   }
   else if( access1 < access2 ) {
      retval = 1;
   }
   else {
      if( is_slot1 != is_slot2 ) {
         retval = ( !is_slot1 && is_slot2 ) ? -1 : 1;
      }
      else if( is_sig1 != is_sig2 ) {
         retval = ( !is_sig1 && is_sig2 ) ? -1 : 1;
      }
   }
   return retval;
}
// end of cmp_access

//*******************************************************************
// swap_items                                                PRIVATE
//*******************************************************************
void ClassesView::swap_items( QTreeWidgetItem* const in_item1, QTreeWidgetItem* const in_item2 )
{
   QTreeWidgetItem tmp = *in_item1;
   *in_item1 = *in_item2;
   *in_item2 = tmp;
}
// end of swap_items
