/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : DataTable.cpp
 * DATE   : 10.03.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "DataTable.h"
#include <QHeaderView>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const QRgb DataTable::GRAY = qRgb( 100, 100, 100 );

//*******************************************************************
// DataTable                                      PUBLIC constructor
//*******************************************************************
DataTable::DataTable(	QWidget* const in_parent,
						const qint32   in_cols,
						const bool     in_set_min_width,
						const qint32   in_n_char_width  )
: QTableWidget    ( in_parent )
, d_X_width       ( fontMetrics().width( 'X' ) )
, d_n_char_width  ( in_n_char_width )
, d_row_height    ( fontMetrics().height() + 2 )
, d_set_min_width ( in_set_min_width )
{
	QPalette plt = palette();
	plt.setColor( QPalette::Highlight, GRAY );
	setPalette( plt );

	QFont fnt = font();
	fnt.setPointSize( fnt.pointSize() - 1 );
	setFont( fnt );

	setColumnCount( in_cols );
	verticalHeader()->hide();
	horizontalHeader()->hide();
	setAlternatingRowColors( true );
	setSelectionBehavior( SelectRows );
	setSelectionMode( SingleSelection );
}
// end of DbgDataTable

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void DataTable::showEvent( QShowEvent* const in_event )
{
	QTableWidget::showEvent( in_event );
	resize_columns();
}
// end of showEvent

//*******************************************************************
// resizeEvent                                     PRIVATE inherited
//*******************************************************************
void DataTable::resizeEvent( QResizeEvent* const in_event )
{
	QTableWidget::resizeEvent( in_event );
	resize_columns();
}
// end of resizeEvent

//*******************************************************************
// edit                                            PRIVATE inherited
//*******************************************************************
bool DataTable::edit( const QModelIndex& in_index, EditTrigger in_trigger, QEvent* const in_event )
{
	return currentColumn() ? QTableWidget::edit( in_index, in_trigger, in_event ) : false;
}
// end of edit

//*******************************************************************
// add_row                                                   PRIVATE
//-------------------------------------------------------------------
// Dodanie nowego wiersza i UTWORZENIE KOMOREK WE WSZYSTKICH KOLUMNACH.
// Numer wiersza wskazany jest przez przyslany parametr.
// Jesli przyslano -1 nowy wiersz nalezy dodac na koncu tablicy.
//*******************************************************************
qint32 DataTable::add_row( const qint32 in_row )
{
	const qint32 row = ( ( in_row < 0 ) || ( in_row > rowCount() ) ? rowCount() : in_row );
	insertRow( row );

	QTableWidgetItem* itm = 0;
	const qint32      n   = columnCount();
		
	for( qint32 i = 0; i < n; ++i ) {
		itm = new QTableWidgetItem();
		itm->setTextAlignment( Qt::AlignBottom );
		setItem( row, i, itm );
	}

	resize_rows();

	return row;
}
// end of add_row

//*******************************************************************
// remove_rows                                                PUBLIC
//*******************************************************************
void DataTable::remove_rows( const qint32 in_new_row_count )
{
	while( in_new_row_count < rowCount() ) {
		removeRow( rowCount() - 1 );
	}
}
// end of remove_rows

//*******************************************************************
// set_item                                                   PUBLIC
//*******************************************************************
void DataTable::set_item( const qint32 in_row, const qint32 in_col, const QString& in_text )
{
	QTableWidgetItem* const itm = item( in_row, in_col );
	if( itm ) itm->setText( in_text.trimmed() );
}
// end of set_item

//*******************************************************************
// get_item                                                   PUBLIC
//*******************************************************************
bool DataTable::get_item( const qint32 in_row, const qint32 in_col, QString& out_text )
{
	out_text.clear();
	const QTableWidgetItem* const itm = item( in_row, in_col );
	if( itm ) out_text = itm->text().trimmed();

	return ( out_text.isEmpty() == false );
}
// end of get_item

//*******************************************************************
// set_tool_tip                                               PUBLIC
//*******************************************************************
void DataTable::set_tool_tip( const qint32 in_row, const qint32 in_col, const QString& in_text )
{
	QTableWidgetItem* const itm = item( in_row, in_col );
	if( itm ) itm->setToolTip( in_text.trimmed() );
}
// end of set_tool_tip

//*******************************************************************
// get_tool_tip                                               PUBLIC
//*******************************************************************
bool DataTable::get_tool_tip( const qint32 in_row, const qint32 in_col, QString& out_text )
{
	out_text.clear();
	const QTableWidgetItem* const itm = item( in_row, in_col );
	if( itm ) out_text = itm->toolTip();

	return ( out_text.isEmpty() == false );
}
// end of get_tool_tip

//*******************************************************************
// resize_columns                                            PRIVATE
//*******************************************************************
void DataTable::resize_columns()
{
	const qint32 n          = columnCount();
	const qint32 min_width  = d_n_char_width * d_X_width;
	const qint32 view_width = viewport()->width();

	for( qint32 i = 0; i < n; ++i ) {
		resizeColumnToContents( i );

		if( i == ( n - 1 ) ) {
			const qint32 total_width = total_columns_width();
			if( total_width < view_width ) {
				setColumnWidth( i, view_width - ( total_width - columnWidth( i ) ) );
			}
		}
		else {
			if( d_set_min_width ) {
				if( columnWidth( i ) < min_width ) {
					setColumnWidth( i, min_width );
				}
			}
		}
	}
}
// end of resize_columns

//*******************************************************************
// total_columns_width                                       PRIVATE
//*******************************************************************
qint32 DataTable::total_columns_width()
{
	qint32 retval = 0;
	const qint32 n = columnCount();

	for( qint32 i = 0; i < n; ++i ) {
		retval += columnWidth( i );
	}
	return retval;
}
// end of total_columns_width

//*******************************************************************
// resize_rows                                               PRIVATE
//*******************************************************************
void DataTable::resize_rows()
{
	for( qint32 i = 0; i < rowCount(); ++i ) {
		if( d_row_height != rowHeight( i ) ) {
			setRowHeight( i, d_row_height );
		}
	}
}
// end of resize_rows
