/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : DbgCallStack.cpp
 * DATE   : 15.02.2007
 *******************************************************************/

/*------- include files:
--------------------------------------------------------------------*/
#include "DbgCallStack.h"
#include "EventsController.h"
#include <QApplication>
#include <QDesktopWidget>
#include <QRegExp>
#include <QHBoxLayout>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const DbgCallStack::CAPTION       = QT_TR_NOOP( "Call stack" );
const QString     DbgCallStack::FRAME_BEGIN   = "frame-begin";
const QString     DbgCallStack::FRAME_END     = "frame-end";
const QString     DbgCallStack::FUNCTION_NAME = "frame-function-name";
const QString     DbgCallStack::SOURCE_FILE   = "frame-source-file";
const QString     DbgCallStack::SOURCE_LINE   = "frame-source-line";
const QString     DbgCallStack::ERROR         = "error";


//*******************************************************************
// DbgCallStack                                          CONSTRUCTOR
//*******************************************************************
DbgCallStack::DbgCallStack( QWidget* const in_parent )
: DbgToolDialog( in_parent )
, d_table( new DbgDataTable( this, 2, false ) )
{
	setWindowTitle( tr( CAPTION ) );

	QHBoxLayout* const mlay = new QHBoxLayout( this );
	mlay->setMargin( 0 );
	mlay->setSpacing( 0 );
	mlay->addWidget( d_table );

	EventsController::instance()->append( this, Event::DBG_CALLSTACK_DATA );
}
// end of DbgCallStack

//*******************************************************************
// ~DbgCallStack                                          DESTRUCTOR
//*******************************************************************
DbgCallStack::~DbgCallStack()
{
	EventsController::instance()->remove( this );
}
// end of ~DbgCallStack

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void DbgCallStack::showEvent( QShowEvent* const in_event )
{
	DbgToolDialog::showEvent( in_event );
	move( qApp->desktop()->screenGeometry().width() - size().width() - 4 - 30, 50 );

	EventsController::instance()->send_event( Event::DBG_CALLSTACK_START );
	EventsController::instance()->send_event( Event::DBG_CALLSTACK_READ );
}
// end of showEvent

//*******************************************************************
// closeEvent                                      PRIVATE inherited
//*******************************************************************
void DbgCallStack::closeEvent( QCloseEvent* const in_event )
{
	EventsController::instance()->send_event( Event::DBG_CALLSTACK_STOP );
	DbgToolDialog::closeEvent( in_event );
}
// end of closeEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void DbgCallStack::customEvent( QEvent* const in_event )
{
	Event* const event = dynamic_cast< Event* >( in_event );
	const qint32 type = static_cast<qint32>( event->type() );
         
	switch( type ) {
		case Event::DBG_CALLSTACK_DATA:
			parse_data( event );
			break;
	}
}
// end of customEvent

//*******************************************************************
// parse_data                                                PRIVATE
//*******************************************************************
void DbgCallStack::parse_data( const Event* const in_event )
{
	d_table->get_prv_data();
	const QStringList data = in_event->d_data[ 0 ].toStringList();

	qint32 row = 0;
	QStringList lines;
	bool is_frame = false;

	QStringList::const_iterator it = data.begin();
	const QStringList::const_iterator end = data.end();
	while( it != end ) {
		if( !is_frame ) {
			if( (*it).startsWith( FRAME_BEGIN ) ) {
				lines << FRAME_BEGIN;
				is_frame = true;
			}
		}
		else {
			if( (*it).contains( ERROR ) ) {
				break;
			}
			if( (*it).startsWith( FRAME_END ) ) {
				lines << FRAME_END;
				set_items( lines );
				lines.clear();
				is_frame = false;
				++row;
			}
			else {
				lines << (*it);
			}
		}
		++it;
	}

	d_table->remove_rows( row );
	d_table->resize_columns();
}
// end of parse_data

//*******************************************************************
// set_items                                                 PRIVATE
//*******************************************************************
void DbgCallStack::set_items( const QStringList& in_lines )
{
	qint32  idx      = -1;
	qint32  nr       = -1;
	QString function = "";
	QString file     = "";
	QString tmp      = "";

	if( ( idx = in_lines.indexOf( FRAME_BEGIN ) ) != -1 ) {
		tmp = in_lines[ idx + 1 ];
		nr = tmp.remove( 0, 1 ).toInt();   // usuwamy # poprzedzajacy numer
		if( ( idx = in_lines.indexOf( FUNCTION_NAME, idx ) ) != -1 ) {
			function = in_lines[ idx + 1 ];
			if( ( idx = in_lines.indexOf( SOURCE_FILE, idx ) ) != -1 ) {
				file = in_lines[ idx + 1 ];
				if( ( idx = in_lines.indexOf( SOURCE_LINE, idx ) ) != -1 ) {
					file += ':';
					file += in_lines[ idx + 1 ];
				}
			}
		}
	}

	const qint32 row = ( nr < d_table->rowCount() ) ? nr : d_table->add_row( nr );
	d_table->set_item( row, 0, function );
	d_table->set_item( row, 1, file );
	d_table->colors( row );
}
// end of set_item
