/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : DbgMemory.cpp
 * DATE   : 21.02.2007
 *******************************************************************/

/*------- include files:
--------------------------------------------------------------------*/
#include "DbgMemory.h"
#include "EventsController.h"
#include <QApplication>
#include <QDesktopWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QIntValidator>
#include <QRegExpValidator>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const DbgMemory::CAPTION       = QT_TR_NOOP( "Memory" );
const char* const DbgMemory::ADDRESS       = QT_TR_NOOP( "&Address:" );
const char* const DbgMemory::HOW_MANY      = QT_TR_NOOP( "How &many:" );
const char* const DbgMemory::HEX           = QT_TR_NOOP( "&Hex" );
const char* const DbgMemory::DEC           = QT_TR_NOOP( "&Dec" );
const char* const DbgMemory::CHAR          = QT_TR_NOOP( "&Char" );
const QString     DbgMemory::REGEXP_MUSTER = "0x[0-9a-fA-F]+";
const QString     DbgMemory::POST_PROMPT   = "post-prompt";
const QString     DbgMemory::PRE_PROMPT    = "pre-prompt";
const QString     DbgMemory::ERROR         = "error";


//*******************************************************************
// DbgMemory                                             CONSTRUCTOR
//*******************************************************************
DbgMemory::DbgMemory( QWidget* const in_parent )
: DbgToolDialog    ( in_parent )
, d_addr           ( new QLineEdit() )
, d_how_many       ( new QLineEdit() )
, d_table          ( new DbgDataTable( this, 9, true, 1 ) )
, d_regexp         ( REGEXP_MUSTER )
, d_addr_value     ( 0 )
, d_how_many_value ( 0 )
, d_is_hex         ( false )
, d_is_dec         ( false )
, d_is_char        ( true )
{
	setWindowTitle( tr( CAPTION ) );

	QVBoxLayout* const mlay = new QVBoxLayout( this );
	mlay->setMargin( 0 );
	mlay->setSpacing( 0 );
	//------------------------------------------------------------------
	QGridLayout* const ilay = new QGridLayout;
	ilay->setMargin( 4 );
	ilay->setSpacing( 4 );

	QLabel* const addr_lbl = new QLabel( tr( ADDRESS ) );
	addr_lbl->setBuddy( d_addr );
	d_addr->setValidator( new QRegExpValidator( d_regexp, d_addr ) );
	ilay->addWidget( addr_lbl, 0, 0 );
	ilay->addWidget( d_addr, 0, 1 );
		
	QLabel* const how_many_lbl = new QLabel( tr( HOW_MANY ) );
	how_many_lbl->setBuddy( d_how_many );
	d_how_many->setValidator( new QIntValidator( 1, 128, d_how_many ) );
	ilay->addWidget( how_many_lbl, 0, 2 );
	ilay->addWidget( d_how_many, 0, 3 );

	mlay->addLayout( ilay );
	//------------------------------------------------------------------
	mlay->addWidget( d_table );
	//------------------------------------------------------------------
	QHBoxLayout* const blay = new QHBoxLayout;

	QPushButton* const hex_btn = new QPushButton( tr( HEX ) );
	QPushButton* const dec_btn = new QPushButton( tr( DEC ) );
	QPushButton* const char_btn = new QPushButton( tr( CHAR ) );
	blay->addWidget( hex_btn );
	blay->addWidget( dec_btn );
	blay->addWidget( char_btn );

	mlay->addLayout( blay );
	//------------------------------------------------------------------

	EventsController::instance()->append( this, Event::DBG_MEMORY_DATA );

	connect( d_addr    , SIGNAL( returnPressed() ), this, SLOT( addr_ret_slot()     ) );
	connect( d_how_many, SIGNAL( returnPressed() ), this, SLOT( how_many_ret_slot() ) );
	connect( hex_btn   , SIGNAL( clicked()       ), this, SLOT( hex_slot()          ) );
	connect( dec_btn   , SIGNAL( clicked()       ), this, SLOT( dec_slot()          ) );
	connect( char_btn  , SIGNAL( clicked()       ), this, SLOT( char_slot()         ) );
}
// end of DbgCallStack

//*******************************************************************
// ~DbgMemory                                             DESTRUCTOR
//*******************************************************************
DbgMemory::~DbgMemory()
{
	EventsController::instance()->remove( this );
}
// end of ~DbgRegisters

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void DbgMemory::showEvent( QShowEvent* const in_event )
{
	DbgToolDialog::showEvent( in_event );
	move( qApp->desktop()->screenGeometry().width() - size().width() - 4 - 120, 200 );
	d_addr->setFocus( Qt::OtherFocusReason );

	EventsController::instance()->send_event( Event::DBG_MEMORY_START );
}
// end of showEvent

//*******************************************************************
// closeEvent                                      PRIVATE inherited
//*******************************************************************
void DbgMemory::closeEvent( QCloseEvent* const in_event )
{
	EventsController::instance()->send_event( Event::DBG_MEMORY_STOP );
	DbgToolDialog::closeEvent( in_event );
}
// end of closeEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void DbgMemory::customEvent( QEvent* const in_event )
{
	Event* const event = dynamic_cast< Event* >( in_event );
	const qint32 type = static_cast<qint32>( event->type() );
         
	switch( type ) {
		case Event::DBG_MEMORY_DATA:
			parse_data( event );
			break;
	}
}
// end of customEvent

//*******************************************************************
// addr_ret_slot                                        PRIVATE slot
//*******************************************************************
void DbgMemory::addr_ret_slot()
{
	d_how_many->setFocus( Qt::TabFocusReason );
}
// end of addr_ret_slot

//*******************************************************************
// how_many_ret_slot                                    PRIVATE slot
//*******************************************************************
void DbgMemory::how_many_ret_slot()
{
	d_addr_value     = d_addr->text().toUInt( 0, 16 );
	d_how_many_value = d_how_many->text().toUInt();
	if( d_addr_value && d_how_many_value ) {
		EventsController::instance()->send_event( Event::DBG_MEMORY_READ, d_how_many_value, d_addr_value );
	}
}
// end of how_many_ret_slot

//*******************************************************************
// parse_data                                                PRIVATE
//*******************************************************************
void DbgMemory::parse_data( Event* const in_event )
{
	const QStringList in_data = in_event->d_data[ 0 ].toStringList();

	if( !in_data.contains( ERROR ) ) {
		QStringList data;
		QString text = "";
	
		QStringList::const_iterator it = in_data.begin();
		const QStringList::const_iterator end = in_data.end();
		while( it != end ) {
			if( POST_PROMPT == (*it ) ) {
				++it;
				continue;
			}
			if( PRE_PROMPT == (*it) ) {
				break;
			}
			if( (*it).startsWith( "0x" ) ) {
				if( !text.isEmpty() ) data << text;
				text = (*it);
			}
			else {
				text += (*it);
			}
			++it;
		}
		if( !text.isEmpty() ) data << text;

		display_data( data );
		d_table->remove_rows( data.size() );
	}

	EventsController::instance()->send_event( Event::DBG_DISABLE_RUN_REQUEST );
}
// end of parse_data

//*******************************************************************
// diasplay_data                                             PRIVATE
//*******************************************************************
void DbgMemory::display_data( const QStringList& in_data )
{
	d_is_hex = d_is_dec = false;
	d_is_char = true;

	qint32 row = 0;
	qint32 idx = 0;
	QChar  chr = '\0';
	QStringList tokens;

	d_table->remove_rows( 0 );

	QStringList::const_iterator it = in_data.begin();
	const QStringList::const_iterator end = in_data.end();
	while( it != end ) {
		tokens = (*it).simplified().split( ' ', QString::SkipEmptyParts );
		if( !tokens.empty() ) {
			const qint32 n = ( tokens.size() > 9 ) ? 9 : tokens.size();
			row = ( idx < d_table->rowCount() ) ? idx : d_table->add_row( idx );
			for( qint32 col = 0; col < n; ++col ) {
				if( col ) {
					chr = tokens[ col ].toUInt();
					if( !chr.isPrint() ) chr = '.';
					d_table->set_item( row, col, chr );
				}
				else {
					d_table->set_item( row, col, tokens[ col ] );
				}
			}
			d_table->resize_columns();
		}
		++idx;
		++it;
	}
}
// end of display_data

//*******************************************************************
// hex_slot                                             PRIVATE slot
//*******************************************************************
void DbgMemory::hex_slot()
{
	if( !d_is_hex ) {
		const qint32 rows_number = d_table->rowCount();
		const qint32 cols_number = d_table->columnCount();
		QString text;
		quint32 code;
	
		for( qint32 row = 0; row < rows_number; ++row ) {
			for( qint32 col = 1; col < cols_number; ++col ) {
				d_table->get_item( row, col, text );
				code = ( d_is_char ) ? static_cast< unsigned char >( text[ 0 ].toAscii() )
									 : text.toUInt( 0, 10 );
				d_table->set_item( row, col, "0x" + QString::number( code, 16 ) );
			}
			d_table->resize_columns();
		}
		d_is_hex = true;
		d_is_dec = d_is_char = false;
	}
}
// end of hex_slot

//*******************************************************************
// dec                                                  PRIVATE slot
//*******************************************************************
void DbgMemory::dec_slot()
{
	if( !d_is_dec ) {
		const qint32 rows_number = d_table->rowCount();
		const qint32 cols_number = d_table->columnCount();
		QString text;
		quint32 code;
	
		for( qint32 row = 0; row < rows_number; ++row ) {
			for( qint32 col = 1; col < cols_number; ++col ) {
				d_table->get_item( row, col, text );
				code = ( d_is_char ) ? static_cast< unsigned char >( text[ 0 ].toAscii() )
									 : text.toUInt( 0, 16 );
				d_table->set_item( row, col, QString::number( code ) );
			}
			d_table->resize_columns();
		}
		d_is_dec = true;
		d_is_hex = d_is_char = false;
	}
}
// end of dec_slot

//*******************************************************************
// char_slot                                            PRIVATE slot
//*******************************************************************
void DbgMemory::char_slot()
{
	if( !d_is_char ) {
		const qint32 rows_number = d_table->rowCount();
		const qint32 cols_number = d_table->columnCount();
		QString text;
		quint32 code;
	
		for( qint32 row = 0; row < rows_number; ++row ) {
			for( qint32 col = 1; col < cols_number; ++col ) {
				d_table->get_item( row, col, text );
				code = ( d_is_dec ) ? text.toUInt( 0, 10 ) : text.toUInt( 0, 16 );
				QChar chr( code );
				if( !chr.isPrint() ) chr = '.';
				d_table->set_item( row, col, chr );
			}
			d_table->resize_columns();
		}
		d_is_char = true;
		d_is_hex = d_is_dec = false;
	}
}
// end of char_slot
