/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR  : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT : Cobras
 * FILE    : FilesView.cpp
 * DATE    : 3.09.2006
 * HISTORY :
 *   17/18.10.2006 - calkowita przebudowa kodu. Wykorzystujemy
 *                   klase QTreeWidget/QTreeWidgetItem.
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "FilesView.h"
#include "QBtParser.h"
#include "Config.h"
#include "EventsController.h"
#include <QMenu>
#include <QFileInfo>
#include <QKeyEvent>
#include <QMouseEvent>
#include <QHeaderView>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const FilesView::SOURCES        = QT_TR_NOOP( "Sources" );
const char* const FilesView::HEADERS        = QT_TR_NOOP( "Headers" );
const char* const FilesView::FORMS          = QT_TR_NOOP( "Forms" );
const char* const FilesView::RESOURCES      = QT_TR_NOOP( "Resources" );
const char* const FilesView::SUBMENU_OPEN   = QT_TR_NOOP( "&Open" );
const char* const FilesView::SUBMENU_RENAME = QT_TR_NOOP( "Re&name" );
const char* const FilesView::SUBMENU_REMOVE = QT_TR_NOOP( "Remo&ve" );


//*******************************************************************
// FilesView                                             CONSTRUCTOR
//*******************************************************************
FilesView::FilesView( QWidget* const in_parent )
: QTreeWidget   ( in_parent )
, d_root_item   ( 0 )
, d_src_item    ( 0 )
, d_hdr_item    ( 0 )
, d_ui_item     ( 0 )
, d_qrc_item    ( 0 )
, d_tmp_buffer  ( "" )
, d_org_palette ( palette() )
, d_org_font    ( font() )
, d_dir         ( "" )
{
   header()->hide();
   setItemsExpandable( true );
   setRootIsDecorated( true );
   setEditTriggers( QTreeView::NoEditTriggers );
   setColumnCount( 1 );
   setHorizontalScrollMode( ScrollPerPixel );
   
   set_properties();
   
   EventsController::instance()->append( this, Event::PROJECT_OPENED );
   EventsController::instance()->append( this, Event::PROJECT_CLOSED );
   EventsController::instance()->append( this, Event::PROJECT_SRC_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_HDR_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_FRM_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_RSC_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_SRC_ONE_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_HDR_ONE_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_FRM_ONE_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_RSC_ONE_FILE_ADDED );
   EventsController::instance()->append( this, Event::PROJECT_ADDED_FINISH );
   EventsController::instance()->append( this, Event::FILE_IN_PROJECT_RENAMED );
   EventsController::instance()->append( this, Event::FILE_IN_PROJECT_REMOVED );
   EventsController::instance()->append( this, Event::FLTREE_PROP_CHANGED );
   
   connect( this, SIGNAL( itemDoubleClicked( QTreeWidgetItem*, int )), this, SLOT( double_clicked( QTreeWidgetItem* )));
}
// end of FilesView

//*******************************************************************
// ~FilesView                                             DESTRUCTOR
//*******************************************************************
FilesView::~FilesView()
{
   EventsController::instance()->remove( this );
}
// end of ~FilesView

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void FilesView::customEvent( QEvent* const in_event )
{
   Q_CHECK_PTR( in_event );

   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<int>( event->type() );

   switch( type ) {
      case Event::PROJECT_OPENED:
         opened( event );
         break;
      case Event::PROJECT_CLOSED:
         closed( event );
         break;
      case Event::PROJECT_SRC_FILE_ADDED:
         src_file_added( event );
         break;
      case Event::PROJECT_SRC_ONE_FILE_ADDED:
         src_file_added( event );
         if( d_src_item ) d_src_item->sortChildren(  0, Qt::AscendingOrder );
         break;
      case Event::PROJECT_HDR_FILE_ADDED:
         hdr_file_added( event );
         break;
      case Event::PROJECT_HDR_ONE_FILE_ADDED:
         hdr_file_added( event );
         if( d_hdr_item ) d_hdr_item->sortChildren(  0, Qt::AscendingOrder );
         break;
      case Event::PROJECT_FRM_FILE_ADDED:
         ui_file_added( event );
         break;
      case Event::PROJECT_FRM_ONE_FILE_ADDED:
         ui_file_added( event );
         if( d_ui_item ) d_ui_item->sortChildren(  0, Qt::AscendingOrder );
         break;
      case Event::PROJECT_RSC_FILE_ADDED:
         qrc_file_added( event );
         break;
      case Event::PROJECT_RSC_ONE_FILE_ADDED:
         qrc_file_added( event );
         if( d_qrc_item ) d_qrc_item->sortChildren(  0, Qt::AscendingOrder );
         break;
      case Event::PROJECT_ADDED_FINISH:
         added_finish();
         break;
      case Event::FILE_IN_PROJECT_RENAMED:
         file_renamed( event );
         break;
      case Event::FILE_IN_PROJECT_REMOVED:
         file_removed( event );
         break;
      case Event::FLTREE_PROP_CHANGED:
         set_properties();
         break;
   }
}
// end of customView

//*******************************************************************
// keyPressEvent                                   PRIVATE inherited
//*******************************************************************
void FilesView::keyPressEvent( QKeyEvent* const in_event )
{
   Q_CHECK_PTR( in_event );

   QTreeView::keyPressEvent( in_event );
      
   switch( in_event->key() ) {
      case Qt::Key_Enter:
      case Qt::Key_Return:
         double_clicked( currentItem() );
         break;
   }
}
// end of keyPressEvent

//*******************************************************************
// mousePressEvent                                 PRIVATE inherited
//*******************************************************************
void FilesView::mousePressEvent( QMouseEvent* const in_event )
{
   Q_CHECK_PTR( in_event );

   QTreeView::mousePressEvent( in_event );

   if( Qt::RightButton == in_event->button() ) {
      const QTreeWidgetItem* item = currentItem();
      if( item ) {
         if( is_file_item( item ) ) {
            QMenu* const menu = new QMenu( this );
            if( menu ) {
               QAction* const open_action   = menu->addAction( tr( SUBMENU_OPEN ) );
               QAction* const rename_action = menu->addAction( tr( SUBMENU_RENAME ) );
               QAction* const remove_action = menu->addAction( tr( SUBMENU_REMOVE ) );

               connect( open_action  , SIGNAL( triggered() ), this, SLOT( submenu_open()   ));
               connect( rename_action, SIGNAL( triggered() ), this, SLOT( submenu_rename() ));
               connect( remove_action, SIGNAL( triggered() ), this, SLOT( submenu_remove() ));

               menu->exec( in_event->globalPos() );

               disconnect( open_action  , SIGNAL( triggered() ), this, SLOT( submenu_open()   ));
               disconnect( rename_action, SIGNAL( triggered() ), this, SLOT( submenu_rename() ));
               disconnect( remove_action, SIGNAL( triggered() ), this, SLOT( submenu_remove() ));
            
               delete menu;
            }
         }
      }
   }
}
// end of mousePressEvent

//*******************************************************************
// set_properties                                            PRIVATE
//*******************************************************************
void FilesView::set_properties()
{
   setAlternatingRowColors( Config::instance()->fltree_use_altbkg() );
   
   if( Config::instance()->fltree_use_defaults() ) {
      setPalette( d_org_palette );
      setFont( d_org_font );
   }
   else {
      QPalette p( d_org_palette );
      p.setColor( QPalette::Text, QColor::fromRgba( Config::instance()->fltree_font_color() ) );
      p.setColor( QPalette::Base, QColor::fromRgba( Config::instance()->fltree_base_bkg() ) );
      p.setColor( QPalette::AlternateBase, QColor::fromRgba( Config::instance()->fltree_altbase_bkg() ) );
      setPalette( p );
   
      // Ustawienie nowego fontu.
      QFont fnt;
      fnt.fromString( Config::instance()->fltree_font() );
      setFont( fnt );
   }
}
// end set_properties

//*******************************************************************
// opened                                                    PRIVATE
//-------------------------------------------------------------------
// Uzytkownik otworzyl nowy projekt.
// Wykonujemy inicjacje drzewa projektu i tworzymy
// element root z nazwa projektu.
// Parametry eventa:
//   0 - nazwa projektu,
//   1 -sciezka do projektu (zakonczona /).
//*******************************************************************
void FilesView::opened( Event* const in_event )
{
   if( 0 == d_root_item ) {
      d_root_item = new QTreeWidgetItem( this );
      if( d_root_item ) {
         d_dir = in_event->d_data.at( 1 ).toString();
         d_root_item->setText( 0, in_event->d_data.at( 0 ).toString() );
         expandItem( d_root_item );
      }
   }
}
// end of opened

//*******************************************************************
// closed                                                    PRIVATE
//-------------------------------------------------------------------
// Uzytkownik zarzadzil zamkniecie projektu.
// Kasujemy wszystko.
//*******************************************************************
void FilesView::closed( Event* const in_event )
{
   if( d_root_item ) {
      if( d_root_item->text( 0 ) == in_event->d_data.at( 0 ).toString() ) {
         clear();
         d_dir.clear();
         d_root_item = 0;
         d_src_item  = 0;
         d_hdr_item  = 0;
         d_ui_item   = 0;
         d_qrc_item  = 0;
      }
   }
}
// end of closed

//*******************************************************************
// src_file_added                                            PRIVATE
//-------------------------------------------------------------------
// Dodanie do drzewa JEDNEGO pliku zrodlowego (cxx, c, cpp).
//*******************************************************************
void FilesView::src_file_added( Event* const in_event )
{
   if( !d_src_item ) {
      d_src_item = new QTreeWidgetItem( d_root_item );
      if( d_src_item ) {
         d_src_item->setText( 0, tr( SOURCES ) );
         d_src_item->setIcon( 0, QIcon( ":/img/source_cpp.png" ) );
      }
      //d_root_item->sortChildren( 0, Qt::AscendingOrder );
   }
   if( d_src_item ) {
      QTreeWidgetItem* const item = new QTreeWidgetItem( d_src_item );
      if( item ) {
         d_buffer = in_event->d_data.at( 0 ).toString();
         item->setText( 0, Shared::relative_path( d_dir, d_buffer ) );
      }
   }
}
// end of src_file_added

//*******************************************************************
// hdr_file_added                                            PRIVATE
//-------------------------------------------------------------------
// Dodanie do drzewa JEDNEGO pliku naglowkowego (h, hpp ).
//*******************************************************************
void FilesView::hdr_file_added( Event* const in_event )
{
   if( !d_hdr_item ) {
      d_hdr_item = new QTreeWidgetItem( d_root_item );
      if( d_hdr_item ) {
         d_hdr_item->setText( 0, tr( HEADERS ) );
         d_hdr_item->setIcon( 0, QIcon( ":/img/source_h.png" ) );
      }
      //d_root_item->sortChildren( 0, Qt::AscendingOrder );
   }
   if( d_hdr_item ) {
      QTreeWidgetItem* const item = new QTreeWidgetItem( d_hdr_item );
      if( item ) {
         d_buffer = in_event->d_data.at( 0 ).toString();
         item->setText( 0, Shared::relative_path( d_dir, d_buffer )  );
      }
   }
}
// end of hdr_file_added

//*******************************************************************
// ui_file_added                                             PRIVATE
//-------------------------------------------------------------------
// Dodanie do drzewa JEDNEGO pliku ui.
//*******************************************************************
void FilesView::ui_file_added( Event* const in_event )
{
   if( !d_ui_item ) {
      d_ui_item = new QTreeWidgetItem( d_root_item );
      if( d_ui_item ) {
         d_ui_item->setText( 0, tr( FORMS ) );
      }
      //d_root_item->sortChildren( 0, Qt::AscendingOrder );
   }
   if( d_ui_item ) {
      QTreeWidgetItem* const item = new QTreeWidgetItem( d_ui_item );
      if( item ) {
         d_buffer = in_event->d_data.at( 0 ).toString();
         item->setText( 0, Shared::relative_path( d_dir, d_buffer ) );
      }
   }
}
// end of ui_file_added

//*******************************************************************
// qrc_file_added                                            PRIVATE
//-------------------------------------------------------------------
// Dodanie do drzewa JEDNEGO pliku zasobow (qrc).
//*******************************************************************
void FilesView::qrc_file_added( Event* const in_event )
{
   if( !d_qrc_item ) {
      d_qrc_item = new QTreeWidgetItem( d_root_item );
      if( d_qrc_item ) {
         d_qrc_item->setText( 0, tr( RESOURCES ) );
      }
      //d_root_item->sortChildren( 0, Qt::AscendingOrder );
   }
   if( d_qrc_item ) {
      QTreeWidgetItem* const item = new QTreeWidgetItem( d_qrc_item );
      if( item ) {
         d_buffer = in_event->d_data.at( 0 ).toString();
         item->setText( 0, Shared::relative_path( d_dir, d_buffer ) );
      }
   }
}
// end of qrc_file_added

//*******************************************************************
// added_finish                                              PRIVATE
//*******************************************************************
void FilesView::added_finish()
{
   if( d_src_item ) d_src_item->sortChildren(  0, Qt::AscendingOrder );
   if( d_hdr_item ) d_hdr_item->sortChildren(  0, Qt::AscendingOrder );
   if( d_ui_item )  d_ui_item->sortChildren (  0, Qt::AscendingOrder );
   if( d_qrc_item ) d_qrc_item->sortChildren(  0, Qt::AscendingOrder );
   d_src_item->setExpanded( true );
}
// end of added_finish

//*******************************************************************
// abs_fpath                                                 PRIVATE
//*******************************************************************
const QString& FilesView::abs_fpath( const QTreeWidgetItem* const in_item )
{
   Q_CHECK_PTR( in_item );
   
   const QFileInfo file_info( d_dir + in_item->text( 0 ) );
   return ( d_buffer = file_info.absoluteFilePath() );
}
// end of abs_fpath

//*******************************************************************
// double_clicked                                       PRIVATE slot
//-------------------------------------------------------------------
// Uzytkownik dwukrotnie kliknal na jakis element.
// Oznacza to, ze nalezy otworzyc plik wskazywany przez ten element.
//*******************************************************************
void FilesView::double_clicked( QTreeWidgetItem* const in_item  )
{
   Q_CHECK_PTR( in_item );
   
   if( d_root_item && in_item ) {
      if( is_file_item( in_item ) ) {
         EventsController::instance()->send_event( Event::DOCUMENT_SELECTED_REQUEST, abs_fpath( in_item ) );
      }
   }
}
// end of double_clicked

//*******************************************************************
// index_for_type                                            PRIVATE
//-------------------------------------------------------------------
// Zwraca adres rodzica dla danego typu pliku.
//*******************************************************************
QTreeWidgetItem* FilesView::parent_for_type( const QString& in_varname ) const
{
   Q_ASSERT( !in_varname.isEmpty() );

   QTreeWidgetItem* retval = 0;

        if( Shared::SRC == in_varname ) retval = d_src_item;
   else if( Shared::HDR == in_varname ) retval = d_hdr_item;
   else if( Shared::UI  == in_varname ) retval = d_ui_item;
   else if( Shared::RSC == in_varname ) retval = d_qrc_item;

   return retval;
}
// end of parent_for_type

//*******************************************************************
// search_child                                              PRIVATE
//*******************************************************************
QTreeWidgetItem* FilesView::search_child( QTreeWidgetItem* const in_parent, const QString& in_fpath )
{
   Q_CHECK_PTR( in_parent );
   Q_ASSERT( !in_fpath.isEmpty() );

   QTreeWidgetItem* retval = 0;

   if( d_root_item && in_parent ) {
      for( qint32 i = 0; i < in_parent->childCount(); ++i ) {
         if( in_fpath == abs_fpath( in_parent->child( i ) ) ) {
            retval = in_parent->child( i );
            break;
         }
      }
   }
   return retval;
}
// end of search_child

//*******************************************************************
// is_file_item                                              PRIVATE
//*******************************************************************
bool FilesView::is_file_item( const QTreeWidgetItem* const in_item ) const
{
   Q_CHECK_PTR( in_item );

   return ( ( in_item != d_src_item )
         && ( in_item != d_hdr_item )
         && ( in_item != d_ui_item  )
         && ( in_item != d_qrc_item ) );
}
// return is_file_item

//*******************************************************************
// file_renamed                                              PRIVATE
//-------------------------------------------------------------------
// Przyslano informacje, ze plik nalezacy do projektu otrzymal inna
// nazwe. Nam pozostaje uaktualnic informacje w drzewie plikow.
//*******************************************************************
void FilesView::file_renamed( Event* const in_event )
{
   Q_CHECK_PTR( in_event );
   
   if( d_root_item ) {
      // Informacje odczytane z eventa.
      const QString old_fpath = in_event->d_data.at( 0 ).toString();
      const QString new_fpath = in_event->d_data.at( 1 ).toString();
      const QString varname   = in_event->d_data.at( 2 ).toString();

      // Parent dla danego rodzaju pliku.
      QTreeWidgetItem* parent_item = parent_for_type( varname );
      if( parent_item ) {
         QTreeWidgetItem* const item = search_child( parent_item, old_fpath );
         if( item ) {
            if( is_file_item( item ) ) {
               item->setText( 0, Shared::relative_path( d_dir, new_fpath ) );
               parent_item->sortChildren(  0, Qt::AscendingOrder );
            }
         }
      }
   }
}
// end of file_renamed

//*******************************************************************
// file_removed                                              PRIVATE
//-------------------------------------------------------------------
// Przyslano informacje, ze plik zostal usuniety z projektu.
//*******************************************************************
void FilesView::file_removed( Event* const in_event )
{
   Q_CHECK_PTR( in_event );

   if( d_root_item ) {
      // Informacje otrzymane z eventa.
      const QString fpath   = in_event->d_data.at( 0 ).toString();
      const QString varname = in_event->d_data.at( 1 ).toString();
      
      // Parent dla danego rodzaju pliku
      QTreeWidgetItem* const parent_item = parent_for_type( varname );
      if( parent_item ) {
         QTreeWidgetItem* const item = search_child( parent_item, fpath );
         if( item ) {
            if( is_file_item( item ) ) {
               parent_item->takeChild( parent_item->indexOfChild( item ) );
            }
         }
      }
   }
}
// end of file_removed

//*******************************************************************
// submenu_open                                         PRIVATE slot
//-------------------------------------------------------------------
// Z submenu uzytkownik wybral polecenie otwarcia pliku.
// Dzialanie to jest, takie same jak dwukrotne klikniecie mysza na
// aktualna pozycje.
//*******************************************************************
void FilesView::submenu_open()
{
   double_clicked( currentItem() );
}
// end of submenu_open

//*******************************************************************
// submenu_rename                                       PRIVATE slot
//-------------------------------------------------------------------
// Z submenu uzytkownik wybral polecenie zmiany nazwy pliku projektu.
// Wysylamy ta informacje w swiat.
//*******************************************************************
void FilesView::submenu_rename()
{
   if( d_root_item ) {
      const QTreeWidgetItem* const item = currentItem();
      if( item ) {
         if( is_file_item( item )  ) {
            EventsController::instance()->send_event( Event::FILE_IN_PROJECT_RENAME, abs_fpath( item ) );
         }
      }
   }
}
// end of submenu_rename

//*******************************************************************
// submenu_remove                                       PRIVATE slot
//-------------------------------------------------------------------
// Z submenu uzytkownik wybral polecenie usuniecia pliku z projektu.
// Wysylamy ta informacje w swiat.
//*******************************************************************
void FilesView::submenu_remove()
{
   if( d_root_item ) {
      const QTreeWidgetItem* item = currentItem();
      if( item ) {
         if( is_file_item( item ) ) {
            EventsController::instance()->send_event( Event::REMOVE_FILE_FROM_PROJECT, abs_fpath( item ) );
         }
      }
   }
}
// end of submenu_remove
