/********************************************************************
 * Copyright (C) 2006, 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Margin.cpp
 * DATE   : 11.11.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Margin.h"
#include "EventsController.h"
#include "Bookmarks.h"
#include "Breakpoints.h"
#include "Config.h"
#include <QPainter>
#include <QTextBlock>
#include <QScrollBar>
#include <QTextLayout>
#include <QColor>
#include <QPalette>
#include <QMenu>
#include <QAction>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const Margin::NEXT_BOOKMARK   = QT_TR_NOOP( "&Next bookmark" );
const char* const Margin::PREV_BOOKMARK   = QT_TR_NOOP( "&Previous bookmark" );
const char* const Margin::TOGGLE_BOOKMARK = QT_TR_NOOP( "&Toggle bookmark" );
const char* const Margin::LIST_BOOKMARK   = QT_TR_NOOP( "&List of all bookmarks" );


//*******************************************************************
// Margin                                                CONSTRUCTOR
//*******************************************************************
Margin::Margin( QWidget* const in_parent ) : QWidget( in_parent )
, width_ ( 0 )
{
   setAutoFillBackground( true );
   set_properties();
}
// end of Margin

//*******************************************************************
// update_geometry                                            PUBLIC
//*******************************************************************
void Margin::update_geometry()
{
   const Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      const QRect r( 0, 0, width(), edt->viewport()->height() );
      setGeometry( r );
   }
}
// end of update_geometry

//*******************************************************************
// set_properties                                             PUBLIC
//*******************************************************************
void Margin::set_properties()
{
   const Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      QPalette p = palette();
      p.setColor( backgroundRole(), Config::instance()->margin_get_bkg_color()  );
      p.setColor( foregroundRole(), Config::instance()->margin_get_font_color() );
      setPalette( p );

      const qint32 mark_size = edt->fontMetrics().height();
      if( mark_size < Config::instance()->bookmark_icon_.height() ) {
         Config::instance()->bookmark_icon_ = Config::instance()->bookmark_icon_.scaled( mark_size, mark_size );
      }
      if( mark_size < Config::instance()->breakpoint_icon_.height() ) {
         Config::instance()->breakpoint_icon_ = Config::instance()->breakpoint_icon_.scaled( mark_size, mark_size );
      }
   }
}
// end of set_properties

//*******************************************************************
// width                                                      PUBLIC
//-------------------------------------------------------------------
// Oblicza aktualna szerokosc marginesu.
// Szerokosc moze zmieniac sie dynamicznie uwzgledniajac liczbe
// linii w dokumencie.
//*******************************************************************
qint32 Margin::width()
{
   qint32 retval = 1 + Config::instance()->bookmark_icon_.width() + 1;
   
   const Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      if( Config::instance()->margin_get_use_numbers() ) {
         qint32 max_number = 0;
         QTextBlock block = edt->document()->end().previous();
         if( block.isValid() ) max_number = block.blockNumber();
      
         // Wyznaczenie dlugosci wyswietlanych liczb.
         const qint32  n      = QString::number( max_number ).length();
         const qint32  nwidth = n * edt->fontMetrics().width( "99", 1 );
         retval += ( nwidth + 2 );
      }
   }
   return ( width_ = retval );
}
// end of width

//*******************************************************************
// on_screen                                                 PRIVATE
//*******************************************************************
bool Margin::on_screen( const qint32 in_top_y, const qint32 in_bottom_y, const qint32 in_y ) const
{
   return ( ( in_y > in_top_y ) && ( in_y < in_bottom_y ) );
}
// end of on_screen

//*******************************************************************
// paintEvent                                      PRIVATE inherited
//*******************************************************************
void Margin::paintEvent( QPaintEvent* )
{
   const Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      QPainter         p( this );
      const QTextBlock cblock = edt->textCursor().block();
      const QString&   fpath = edt->file_path();
            
      // Wspolrzedne gory i dolu widoku (ekranu).
      const qint32 y_top    = edt->verticalScrollBar()->value();
      const qint32 y_bottom = y_top + edt->viewport()->height() - 1;

      QTextBlock block = edt->document()->begin();
      while( block.isValid() ) {
         const int nr = block.blockNumber();
         
         // Wspolrzedne aktualnego bloku.
         const qint32 block_yt     = qRound( block.layout()->position().y() );
         const qint32 block_height = block.layout()->boundingRect().toRect().height();
         const qint32 block_yb     = block_yt + block_height - 1;

         // Jezeli gorna czesc bloku jest ponizej ekranu to przerywamy prace.
         // (Szkoda czasu na robienie czegokolwiek dla niewidocznej czesci edytora.)
         if( block_yt > y_bottom ) {
            break;
         }
         // Jesli chocby ulamek aktualnego bloku zawiera sie w widoku
         // to go musimy obsluguzyc.
         if( on_screen( y_top, y_bottom, block_yt ) || on_screen( y_top, y_bottom, block_yb ) ) {
            const Config* const cfg = Config::instance();
            const LineInfo* const info = dynamic_cast<LineInfo*>( block.userData() );
            if( info ) {
               if( block == cblock ) {
                  const QRect rect( 0, block_yt - y_top + 1, width_, block_height + 1 );
                  p.fillRect( rect, QBrush( cfg->margin_get_sel_color() ) );
               }
               if( cfg->margin_get_use_numbers() ) {
                  const QRect rect( 0, block_yt - y_top + 1, width_ - 3, block_height  );
                  p.drawText( rect, Qt::AlignRight | Qt::AlignBottom, QString::number( nr + 1 ) );
               }
               // Wyswietlenie (ewentualne) ikonek.
               if( Points::SET == Breakpoints::instance()->status( fpath, nr ) ) {
                  p.drawPixmap( 1, block_yt - y_top , cfg->breakpoint_icon_ );
               }
               else {
                  if( Points::SET == Bookmarks::instance()->status( fpath, nr ) ) {
                     p.drawPixmap( 1, block_yt - y_top , cfg->bookmark_icon_ );
                  }
               }   
            }
         }
         block = block.next();
      }
      p.end();
   }
}
// end of paintEvent

//*******************************************************************
// mouseDoubleClickEvent                           PRIVATE inherited
//-------------------------------------------------------------------
// Ustawianie znacznikow (bookmarks).
//*******************************************************************
void Margin::mouseDoubleClickEvent( QMouseEvent* const in_event )
{
   Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      if( Qt::LeftButton == in_event->button() ) {
         QTextBlock block;
         const int y_pos = in_event->y() + edt->verticalScrollBar()->value();
         if( search_text_block( y_pos, block ) ) {
            LineInfo* const info = dynamic_cast<LineInfo*>( block.userData() );
            if( info ) {
               const QString fpath = edt->file_path();
               const int row = block.blockNumber();
               Bookmarks::instance()->change_status( fpath, row, block.text().trimmed() );
               info->bookmark = ( Bookmarks::instance()->status( fpath, row ) == Points::SET );
               edt->set_cursor_position( row + 1, 1 );
               update();
            }
         }
      }
   }
}
// end of mouseDoubleClickEvent

//*******************************************************************
// mousePressEvent                                 PRIVATE inherited
//*******************************************************************
void Margin::mousePressEvent( QMouseEvent* const in_event )
{
   Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      QTextBlock block;
      const qint32 y_pos = in_event->y() + edt->verticalScrollBar()->value();
      if( search_text_block( y_pos, block ) ) {
         edt->set_cursor_position( block.position() );
         if( Qt::RightButton == in_event->button() ) {
            context_menu( in_event->globalPos() );
         }
         else if( Qt::MidButton == in_event->button() ) {
            QTextCursor current_cursor = edt->textCursor();
            if( current_cursor.hasSelection() ) {
               current_cursor.clearSelection();
            }
            else {
               current_cursor.select( QTextCursor::BlockUnderCursor );
            }
            edt->setTextCursor( current_cursor );
         }
      }
   }
}
// end of mousePressEvent

//*******************************************************************
// search_text_block                                         PRIVATE
//-------------------------------------------------------------------
// Wyszukiwanie w tekscie bloku/linii, ktora zawiera w sobie
// przyslana wspolrzedna.
//*******************************************************************
bool Margin::search_text_block ( const qint32 in_y, QTextBlock& out_block ) const
{
   bool retval = false;

   const Editor* const edt = dynamic_cast<Editor*>( parent() );
   if( edt ) {
      // Wspolrzedne gory i dolu widoku zawierajacego tekst w edytorze.
      const qint32 yt_view = edt->verticalScrollBar()->value();
      const qint32 yb_view = yt_view + edt->viewport()->height() - 1;

      QTextBlock block = edt->document()->begin();
      while( block.isValid() ) {
         if( in_y > yt_view ) {
            // Jesli jestesmy juz za widoczna czescia widoku
            // przerywamy prace. I tak juz nic nie ma do pokazania.
            if ( in_y > yb_view ) {
               break;
            }
         
            // Wspolrzedne gory i dolu aktualnego bloku.
            const qint32 yt_block = qRound( block.layout()->position().y() );
            const qint32 yb_block = yt_block + qRound( block.layout()->boundingRect().height() ) - 1;
            // Jezeli przyslana zawiera sie pomiedzy gora i dolem
            // aktualnego bloku, to ten blok jest tym czego szukamy.
            if( ( in_y >= yt_block ) && ( in_y <= yb_block ) ) {
               out_block = block;
               retval = true;
               break;
            }
            block = block.next();
         }
      }
   }
   return retval;
}
// end of search_text_block

//*******************************************************************
// context_menu                                              PRIVATE
//*******************************************************************
void Margin::context_menu( const QPoint& in_point )
{
   QMenu* const menu = new QMenu( this );
   if( menu ) {
      QAction* const next   = menu->addAction( QIcon( ":/img/bookmark_next.png" ),tr( NEXT_BOOKMARK ) );
      QAction* const prev   = menu->addAction( QIcon( ":/img/bookmark_prev.png" ), tr( PREV_BOOKMARK ) );
      menu->addSeparator();
      QAction* const toggle = menu->addAction( QIcon( ":/img/bookmark_toggle.png" ), tr( TOGGLE_BOOKMARK ) );
      menu->addSeparator();
      QAction* const all    = menu->addAction( QIcon( ":/img/bookmark_list.png" ),tr( LIST_BOOKMARK ) );
      
      connect( next  , SIGNAL( triggered() ), this, SLOT( bookmark_next()   ) );
      connect( prev  , SIGNAL( triggered() ), this, SLOT( bookmark_prev()   ) );
      connect( toggle, SIGNAL( triggered() ), this, SLOT( bookmark_toggle() ) );
      connect( all   , SIGNAL( triggered() ), this, SLOT( bookmark_all()    ) );

      menu->exec( in_point );

      disconnect( next  , SIGNAL( triggered() ), this, SLOT( bookmark_next()   ) );
      disconnect( prev  , SIGNAL( triggered() ), this, SLOT( bookmark_prev()   ) );
      disconnect( toggle, SIGNAL( triggered() ), this, SLOT( bookmark_toggle() ) );
      disconnect( all   , SIGNAL( triggered() ), this, SLOT( bookmark_all()    ) );
      
      delete menu;
   }
}
// end of context_menu

//*******************************************************************
// bookmark_next                                             PRIVATE
//*******************************************************************
void Margin::bookmark_next()
{
   EventsController::instance()->send_event( Event::BOOKMARK_NEXT_REQUEST );
}
// end of bookmark_next

//*******************************************************************
// bookmark_prev                                             PRIVATE
//*******************************************************************
void Margin::bookmark_prev()
{
   EventsController::instance()->send_event( Event::BOOKMARK_PREV_REQUEST );
}
// end of bookmark_prev

//*******************************************************************
// bookmark_toggle                                           PRIVATE
//*******************************************************************
void Margin::bookmark_toggle()
{
   EventsController::instance()->send_event( Event::BOOKMARK_TOGGLE_REQUEST );
}
// end of bookmark_toggle

//*******************************************************************
// bookmark_all                                              PRIVATE
//*******************************************************************
void Margin::bookmark_all()
{
   EventsController::instance()->send_event( Event::BOOKMARK_ALL_REQUEST );
}
// end of bookmark_all
