/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectController.cpp
 * DATE   : 21.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectController.h"
#include "Project.h"
#include "Config.h"
#include "QBtFind.h"
#include "GetQmake.h"
#include "EventsController.h"
#include "AddFile.h"
#include "Classer.h"
#include "QBtParser.h"
#include "Debugger.h"
#include "Process.h"
#include "Shared.h"
#include <QTextDocument>
#include <QFile>
#include <QTextStream>
#include <QMessageBox>
#include <QFileDialog>
#include <QApplication>
#include <QTimer>
#include <QInputDialog>
#include <QLineEdit>
#include <QInputDialog>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ProjectController::FIND_CAPTION                = QT_TR_NOOP( "Find" );
const char* const ProjectController::DIALOG_OPEN_PROJECT_CAPTION = QT_TR_NOOP( "Open a project file" );
const char* const ProjectController::PROJECTS_FILTER             = QT_TR_NOOP( "ProjectController files (*.pro);;All files(*.*)" );
const char* const ProjectController::CAPTION                     = QT_TR_NOOP( "Qt Project" );
const char* const ProjectController::CANT_OPEN_FILE              = QT_TR_NOOP( "Can't open file %1\n(%2)" );
const char* const ProjectController::NOT_VALID_PROJECT           = QT_TR_NOOP( "File %1 is not a valid Qt project" );
const char* const ProjectController::PROJECT_IS_OPENED           = QT_TR_NOOP( "You would like open an project.\nBut currently is opened project \'%1\'\nWould you like close him?" );
const char* const ProjectController::ADD_FILE_TO_PROJECT         = QT_TR_NOOP( "Enter the name of project file:" );
const char* const ProjectController::UNKNOWN_FILE_TYPE           = QT_TR_NOOP( "Unknown file type (need extension)." );
const char* const ProjectController::INVALID_EXTENSION           = QT_TR_NOOP( "Invalid file extension" );
const char* const ProjectController::DIR_NOT_EXISTS              = QT_TR_NOOP( "Directory not exists" );
const char* const ProjectController::CANT_CREATE                 = QT_TR_NOOP( "Can't create the file\n(%1)" );
const char* const ProjectController::ALREADY_EXISTS              = QT_TR_NOOP( "This file already exists\n(%1)" );
const char* const ProjectController::CANT_APPEND                 = QT_TR_NOOP( "Can't append this file to project\n(%1)" );
const char* const ProjectController::FILE_TYPE_CHANGED           = QT_TR_NOOP( "Can't change file type" );
const char* const ProjectController::RENAME_PROMPT               = QT_TR_NOOP( "Enter new file name:" );


//*******************************************************************
// ProjectController                                               CONSTRUCTOR
//*******************************************************************
ProjectController::ProjectController( QWidget* const in_parent )
: QObject       ( in_parent )
, d_parent      ( in_parent )
, d_last_prj_dir( QDir::homePath() )
, d_process     ( 0 )
, d_debugger    ( 0 )
{
   EventsController::instance()->append( this, Event::OPEN_PROJECT_REQUEST );
   EventsController::instance()->append( this, Event::CLOSE_PROJECT_REQUEST );
   EventsController::instance()->append( this, Event::PROJECT_PROPERTIES_REQUEST );
   EventsController::instance()->append( this, Event::PROJECT_SELECTED_REQUEST );
   EventsController::instance()->append( this, Event::NEW_PROJECT_REQUEST );
   EventsController::instance()->append( this, Event::ADD_FILE_TO_PROJECT_REQUEST );
   EventsController::instance()->append( this, Event::REMOVE_FILE_FROM_PROJECT );
   EventsController::instance()->append( this, Event::FILE_IN_PROJECT_RENAME );
   EventsController::instance()->append( this, Event::FIND_IN_PROJECT_FILES_REQUEST );
   EventsController::instance()->append( this, Event::RUN_REQUEST );
   EventsController::instance()->append( this, Event::BUILD_REQUEST );
   EventsController::instance()->append( this, Event::REBUILD_REQUEST );
   EventsController::instance()->append( this, Event::BREAK_REQUEST );
   EventsController::instance()->append( this, Event::PROCESS_BROKEN );
   EventsController::instance()->append( this, Event::DBG_RUN_REQUEST );
   EventsController::instance()->append( this, Event::DBG_STOP_REQUEST );
   EventsController::instance()->append( this, Event::DBG_FINISHED );

   d_files.clear();
}
// end of ProjectController

//*******************************************************************
// ~ProjectController                                     DESTRUCTOR
//*******************************************************************
ProjectController::~ProjectController()
{
   EventsController::instance()->remove( this );

   if( d_process ) {
      delete d_process;
      d_process = 0;
   }
   if( d_debugger ) {
      delete d_debugger;
      d_debugger = 0;
   }
}
// end of ~ProjectController

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void ProjectController::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<int>( event->type() );
   
   switch( type ) {
      case Event::OPEN_PROJECT_REQUEST:
         open_project();
         break;
      case Event::PROJECT_SELECTED_REQUEST:
         project_selected( event );
         break;
      case Event::CLOSE_PROJECT_REQUEST:
         close_project();
         break;
      case Event::NEW_PROJECT_REQUEST:
         new_project();
         break;
      case Event::PROJECT_PROPERTIES_REQUEST:
         project_properties();
         break;
      case Event::ADD_FILE_TO_PROJECT_REQUEST:
         add_file_to_project();
         break;
      case Event::REMOVE_FILE_FROM_PROJECT:
         remove_file_from_project( event );
         break;
      case Event::FIND_IN_PROJECT_FILES_REQUEST:
         find_in_project_files();
         break;
      case Event::FILE_IN_PROJECT_RENAME:
         rename( event->d_data.at( 0 ).toString() );
         break;
      case Event::RUN_REQUEST:
         run();
         break;
      case Event::BUILD_REQUEST:
         build();
         break;
      case Event::REBUILD_REQUEST:
         rebuild();
         break;
      case Event::BREAK_REQUEST:
         process_break();
         break;
      case Event::PROCESS_BROKEN:
         process_broken();
         break;
      case Event::DBG_RUN_REQUEST:
         dbg_run();
         break;
      case Event::DBG_STOP_REQUEST:
         dbg_stop();
         break;
      case Event::DBG_FINISHED:
         d_debugger->kill();
         d_debugger->waitForFinished( 1000 );
         delete d_debugger;
         d_debugger = 0;
         break;
   }
}
// end of customEvent

//*******************************************************************
// project_properties                                        PRIVATE
//*******************************************************************
void ProjectController::project_properties()
{
   if( d_project.is_opened() ) {
      if( d_project.change_properties() ) {
         const QString fpath = d_project.path();
         close_project();
         open( fpath );
      }
   }
}
// end of project_properties

//*******************************************************************
// new_project                                               PRIVATE
//*******************************************************************
void ProjectController::new_project()
{
   if( can_open() ) {
      QString fpath;
      if( d_project.create( fpath ) ) {
         open( fpath );
      }
   }
}
// end of new_project

//*******************************************************************
// open_project                                              PRIVATE
//-------------------------------------------------------------------
// Otwarcie jakiegos istniejacego juz projektu.
// Uzytkownik powinien go wskazac przy uzyciu dialogu.
//*******************************************************************
void ProjectController::open_project()
{
   if( can_open() ) {   // sprawdzamy, czy mozna otworzyc nowy projekt?
      const QString fpath = QFileDialog::getOpenFileName( d_parent,
            tr( DIALOG_OPEN_PROJECT_CAPTION ),
            d_last_prj_dir,
            tr( PROJECTS_FILTER ) );

      if( fpath != QString::null ) {
         open( fpath );
      }
   }
}
// end of open_project

//*******************************************************************
// can_open                                                  PRIVATE
//-------------------------------------------------------------------
// Wyjasnienie czy mozna otworzyc nowy projekt.
// Problem powstaje wtedy, gdy jakis projekt juz jest otwarty.
// Jesli tak jest uzytkownik musi zgodzic sie z sugestia,
// zeby zamknac aktualnie otwarty projekt.
//*******************************************************************
bool ProjectController::can_open()
{
   bool retval = true;

   if( d_project.is_opened() ) {
      const qint32 answer =
         QMessageBox::question( 0,
            tr( DIALOG_OPEN_PROJECT_CAPTION ),
            tr( PROJECT_IS_OPENED ).arg( d_project.file_name() ),
            QMessageBox::Cancel | QMessageBox::Escape,
            QMessageBox::Yes | QMessageBox::Default );

      if( QMessageBox::Yes == answer ) close_project();
      else                             retval = false;
   }
   return retval;
}
// end of can_open

//*******************************************************************
// open                                                      PRIVATE
//*******************************************************************
void ProjectController::open( const QString& in_fpath )
{
   Q_ASSERT( !in_fpath.isEmpty() );

   if( d_project.open( in_fpath ) ) {
      EventsController::instance()->send_event( Event::PROJECT_OPENED, d_project.file_name(), d_project.dir() );

      QStringList list_of_vars;
      if( d_project.src( list_of_vars ) ) add_items( list_of_vars, Event::PROJECT_SRC_FILE_ADDED );
      if( d_project.hdr( list_of_vars ) ) add_items( list_of_vars, Event::PROJECT_HDR_FILE_ADDED );
      if( d_project.ui ( list_of_vars ) ) add_items( list_of_vars, Event::PROJECT_FRM_FILE_ADDED );
      if( d_project.qrc( list_of_vars ) ) add_items( list_of_vars, Event::PROJECT_RSC_FILE_ADDED );

      list_of_vars.clear();
      qStableSort( d_files.begin(), d_files.end() );

      EventsController::instance()->send_event( Event::PROJECT_ADDED_FINISH );
      Classer::instance()->parse( d_files );
   }
   else {
      d_project.close();
      Shared::critical( d_parent,
         tr( CAPTION ),
         tr( NOT_VALID_PROJECT ).arg( in_fpath ) );
   }
}
// end of open

//*******************************************************************
// add_items                                                 PRIVATE
//*******************************************************************
void ProjectController::add_items( const QStringList& in_vr, const int in_event_type )
{
   QFileInfo file_info;
   const QString dir = d_project.dir();

   QStringList::const_iterator it = in_vr.begin();
   const QStringList::const_iterator end = in_vr.end();
   while( it != end ) {
      d_files << *it;
      EventsController::instance()->send_event( in_event_type, *it );
      ++it;
   }
}
// end of add_items

//*******************************************************************
// remove_file_from_project                                  PRIVATE
//*******************************************************************
void ProjectController::remove_file_from_project( Event* const in_event )
{
   if( in_event && d_project.is_opened() ) {
      const QString fpath = in_event->d_data.at( 0 ).toString();
      const int idx = d_files.indexOf( fpath );
      if( idx != -1 ) {
         const QString varname = d_project.remove_file( fpath );
         if( !varname.isEmpty() ) {
            d_files.removeAt( idx );
            EventsController::instance()->send_event( Event::FILE_IN_PROJECT_REMOVED, fpath, varname );
            d_project.save();
         }
      }
   }
}
// end of remove_file_from_project

//*******************************************************************
// rename                                                    PRIVATE
//*******************************************************************
void ProjectController::rename( const QString& in_fpath )
{
   Q_ASSERT( !in_fpath.isEmpty() );
   
   if( d_project.is_opened() ) {
      QStringList::iterator it = qBinaryFind( d_files.begin(), d_files.end(), in_fpath );
      if( it != d_files.end() ) {
         const QFileInfo file_info( in_fpath );
         
         const QString fdir  = file_info.absolutePath();
         const QString fext  = file_info.suffix();
               QString fname = file_info.fileName();
         
         fname.resize( fname.length() - fext.length() - 1 );

         bool ok;
         const QString new_fname = QInputDialog::getText( d_parent,
                                                          tr( CAPTION ),
                                                          tr( RENAME_PROMPT ),
                                                          QLineEdit::Normal,
                                                          fname, &ok );
         if( ok ) {
            QString new_fpath = fdir;
            new_fpath += '/';
            new_fpath += new_fname;
            new_fpath += '.';
            new_fpath += fext;
            
            const QString varname = d_project.rename_file( in_fpath, new_fpath );
            if( !varname.isEmpty() ) {
               *it = new_fpath;
               if( QFile::rename( in_fpath, new_fpath ) ) {
                  EventsController::instance()->send_event( Event::FILE_IN_PROJECT_RENAMED, in_fpath, new_fpath, varname );
                  d_project.save();
               }
            }
            else {
               Shared::critical( d_parent, tr( CAPTION ), tr( FILE_TYPE_CHANGED ) );
            }
            qStableSort( d_files.begin(), d_files.end() );
         }
      }
   }
}
// end of rename

//*******************************************************************
// project_selected                                          PRIVATE
//-------------------------------------------------------------------
// Otwarcie kiedys otwartego juz projektu.
// Uzytkownik wybral go w menu 'File' z listy ostatnio
// otwartych projektow.
//*******************************************************************
void ProjectController::project_selected( Event* const in_event )
{
   if( can_open() ) {   // najpierw sprawdzamy, czy mozna otworzyc nowy projekt
      open( in_event->d_data.at( 0 ).toString() );
   }
}
// end of project_selected

//*******************************************************************
// close_project                                             PRIVATE
//-------------------------------------------------------------------
// Zamkniecie aktualnie otwartego projektu.
//*******************************************************************
void ProjectController::close_project()
{
   EventsController::instance()->send_event( Event::PROJECT_CLOSED, d_project.file_name(), d_project.dir() );
   d_project.close();
   d_files.clear();
}
// end of close_project

//*******************************************************************
// add_file_to_project                                       PRIVATE
//*******************************************************************
void ProjectController::add_file_to_project()
{
   if( d_project.is_opened() ) {
      QString fpath         = "";
      QString file1         = "";
      QString file2         = "";
      bool    src_selected  = false;
      bool    hdr_selected  = false;

      AddFile* const dialog = new AddFile( d_parent, d_project.dir() );
      if( dialog ) {
         if( 2 == dialog->exec() ) {
            fpath        = dialog->selected_file().trimmed();   // pelna(!) sciezka do pliku
            src_selected = dialog->source_is_checked();
            hdr_selected = dialog->header_is_checked();
         }
         delete dialog;
      }
   
      //  Ustalenie pelnych sciezek do nowych plikow.
      if( !fpath.isEmpty() ) {
         const QString fext = Shared::extension( fpath );
         if( fext.isEmpty() ) {
            if( src_selected ) file1 = fpath + ".cpp";
            if( hdr_selected ) file2 = fpath + ".h";
         }
         else if( Shared::is_src_ext( fext ) ) {
            file1 = fpath;
            if( hdr_selected ) file2 = Shared::remove_extension( fpath ) + ".h";
         }
         else if( Shared::is_hdr_ext( fext ) ) {
            file2 = fpath;
            if( src_selected ) file1 = Shared::remove_extension( fpath ) + ".cpp";
         }
         else if( Shared::is_ui_ext( fext ) ) {
            file1 = fpath;
         }
         else if( Shared::is_qrc_ext( fext ) ) {
            file1 = fpath;
         }      
      }

      // Dodanie plikow do projektu i tworzenie pustych plikow
      if( !file1.isEmpty() ) add_file( file1 );
      if( !file2.isEmpty() ) add_file( file2 );
      d_project.save();
      qStableSort( d_files.begin(), d_files.end() );
   }
}
// end of add_file_to_project

//*******************************************************************
// add_file                                                  PRIVATE
//*******************************************************************
bool ProjectController::add_file( const QString& in_fpath )
{
   bool retval = false;

   QStringList::iterator it = qBinaryFind( d_files.begin(), d_files.end(), in_fpath );

   if( it != d_files.end() ) {
      Shared::critical( d_parent, tr( CAPTION ), tr( ALREADY_EXISTS ).arg( in_fpath ) );
   }
   else {
      const QString varname = d_project.add_file( in_fpath );
      if( varname.isEmpty() ) {
         Shared::critical( d_parent, tr( CAPTION ), tr( CANT_APPEND ).arg( in_fpath ) );
      }
      else {
         retval = true;
         d_files << in_fpath;
         if( !QFile::exists( in_fpath ) ) {
            Shared::create_empty_file( in_fpath );
         }
         update_file_view( varname, in_fpath );
      }
   }
   return retval;
}
// end of add_file

//*******************************************************************
// update_file_view                                          PRIVATE
//-------------------------------------------------------------------
// Wyslanie okreslonego eventa w celu dodania wskazanego pliku
// do drzewa plikow projektow.
//*******************************************************************
void ProjectController::update_file_view( const QString& in_varname, const QString& in_fpath  )
{
   qint32 event_type = -1;
        if( "SOURCES"   == in_varname ) event_type = Event::PROJECT_SRC_ONE_FILE_ADDED;
   else if( "HEADERS"   == in_varname ) event_type = Event::PROJECT_HDR_ONE_FILE_ADDED;
   else if( "FORMS"     == in_varname ) event_type = Event::PROJECT_FRM_ONE_FILE_ADDED;
   else if( "RESOURCES" == in_varname ) event_type = Event::PROJECT_RSC_ONE_FILE_ADDED;

   if( event_type != -1 ) {
      EventsController::instance()->send_event( event_type, in_fpath );
   }
}
// end of update_file_view

//*******************************************************************
// find_in_project_files                                     PRIVATE
//*******************************************************************
void ProjectController::find_in_project_files()
{
   QString find_text( "" );
   QRegExp expr( "" );
   QTextDocument::FindFlags find_flags = 0;

   // Za pomoca wprowadzonych przez uzytkownika
   // ustalamy wzorzec poszukiwan.
   QBtFind dialog( 0, "" );
   dialog.find_in_project();
   
   if( QBtFind::Accepted == dialog.exec() ) {
      dialog.input_text( find_text );
      if( dialog.is_whole() ) {
         find_text.insert( 0, "\\b" );
         find_text += "\\b";
      }
      expr.setPattern( find_text );
      if( dialog.is_sensitive() ) {
         expr.setCaseSensitivity( Qt::CaseSensitive );
      }
   }

   // Jesli wzorzec jest w porzadku
   // rozpocznij poszukiwania.
   if( !find_text.isEmpty() ) {
      QApplication::setOverrideCursor( Qt::WaitCursor );
      find_in_files( expr );
      QApplication::restoreOverrideCursor();
   }
}
// end of find_in_project_files

//*******************************************************************
// find_in_files                                             PRIVATE
//*******************************************************************
void ProjectController::find_in_files( const QRegExp& in_expr )
{
   QStringList::const_iterator it = d_files.begin();
   const QStringList::const_iterator end = d_files.end();
   while( it != end ) {
      find_in_this_file( *it, in_expr );
      ++it;
   }
}
// end of find_in_files

//*******************************************************************
// find_in_this_file                                         PRIVATE
//*******************************************************************
void ProjectController::find_in_this_file( const QString& in_file_path, const QRegExp& in_expr )
{
   QString line;
   qint32 line_number = 0;

   QFile file( in_file_path );
   if( file.open( QIODevice::ReadOnly ) ) {
      QTextStream in( &file );
      while( !in.atEnd() ) {
         ++line_number;
         line = in.readLine();
         if( line.indexOf( in_expr ) != -1 ) {
            EventsController::instance()->send_event( Event::SEARCH_BROWSER_ITEM, in_file_path, line_number, line );
         }
      }
      file.close();
   }
   else {
      QMessageBox::warning( 0,
         tr( FIND_CAPTION ),
         tr( CANT_OPEN_FILE ).arg( file.fileName() ).arg( file.errorString() ));
   }
}
// end of find_in_this_file


//###################################################################
//#                                                                 #
//#                           Q M A K E                             #
//#                                                                 #
//###################################################################

//*******************************************************************
// before_qmake                                              PRIVATE
//*******************************************************************
void ProjectController::before_qmake( QStringList& out_cmd )
{
   QStringList result;

   const QStringList& strings = Config::instance()->qmake_before();
   if( !strings.empty() ) {
      QString     txt = "";
      QString     str = "";

      QStringList src, hdr;
      d_project.src( src );
      d_project.hdr( hdr );

      {   // skanowanie wzgledem plikow zrodlowych
         QStringList::const_iterator it = strings.begin();
         const QStringList::const_iterator end = strings.end();
         while( it != end ) {
            txt = *it;
            if( !txt.isEmpty() ) {
               if( txt.contains( "%c" ) ) {
                  QStringList::const_iterator src_it = src.begin();
                  const QStringList::const_iterator src_end = src.end();
                  while( src_it != src_end ) {
                     str = txt;
                     result << str.replace( "%c", *src_it );
                     ++src_it;
                  }
               }
               else {
                  result << txt;
               }
            }
            ++it;
         }
      }

      {   // skanowanie wzgledem plikow naglowkowych
         QStringList tmp = result;
         result.clear();
         QStringList::const_iterator it = tmp.begin();
         const QStringList::const_iterator end = tmp.end();
         while( it != end ) {
            txt = *it;
            if( !txt.isEmpty() ) {
               if( txt.contains( "%h" ) ) {
                  QStringList::const_iterator hdr_it = hdr.begin();
                  const QStringList::const_iterator hdr_end = hdr.end();
                  while( hdr_it != hdr_end ) {
                     str = txt;
                     result << str.replace( "%h", *hdr_it );
                     ++hdr_it;
                  }
               }
               else {
                  result << txt;
               }
            }
            ++it;
         }
      }
   }

   out_cmd = result;
}
// end of before_qmake

//*******************************************************************
// after_qmake                                               PRIVATE
//*******************************************************************
void ProjectController::after_qmake( QStringList& out_cmd )
{
   const QStringList& strings = Config::instance()->qmake_after();
   if( !strings.empty() ) {
      QString txt = "";
      QStringList::const_iterator it = strings.begin();
      const QStringList::const_iterator end = strings.end();
      while( it != end ) {
         txt = *it;
         if( !txt.isEmpty() ) {
            txt.replace( "%p", d_project.name() );
            out_cmd << txt;
         }
         ++it;
      }
   }
}
// end of after_qmake

//*******************************************************************
// run                                                       PRIVATE
//*******************************************************************
void ProjectController::run()
{
   if( d_project.is_opened() && ( 0 == d_process ) ) {
      QString fpath = d_project.dir();
      
      const QString dest_dir = d_project.dest_dir();
      if( !dest_dir.isEmpty() ) {
         // Nalezy rozroznic dwa przypadki.
         // 1. DESTDIR zawiera abolutna sciezke gdzie jest program wynikowy,
         // 2. DESTDIR zawiera sciezke wzgledna - odnosi sie ona do sciezki projektu
         if( dest_dir.startsWith( "/" ) ) {
            fpath = dest_dir;
         }
         else {
            const QDir dir( fpath + dest_dir );
            fpath = dir.absolutePath();
         }
         Shared::complete_dir( fpath );
      }

      fpath += d_project.name();
      fpath += " &";

      d_process = new Process();
      if( d_process ) {
         QStringList cmd;
         before_qmake( cmd );
         cmd << "make";
         after_qmake( cmd );
         cmd << fpath;
         d_process->run( d_project.dir(), cmd );
      }
   }
}
// end of run

//*******************************************************************
// build                                                     PRIVATE
//*******************************************************************
void ProjectController::build()
{
   if( d_project.is_opened() && ( 0 == d_process ) ) {
      d_process = new Process();
      if( d_process ) {
         QStringList cmd;
         before_qmake( cmd );
         cmd << "make";
         after_qmake( cmd );
         d_process->run( d_project.dir(), cmd );
      }
   }
}
// end of build

//*******************************************************************
// build                                                     PRIVATE
//*******************************************************************
void ProjectController::rebuild()
{
   if( d_project.is_opened() && ( 0 == d_process )  ) {
      if( Config::instance()->qmake_path().isEmpty() ) {
         GetQmake* const dialog = new GetQmake( 0 );
         if( dialog ) {
            dialog->exec();
            delete dialog;
         }
      }

      if( !Config::instance()->qmake_path().isEmpty() ) {
         d_process = new Process();
         if( d_process ) {
            QStringList cmd;
            before_qmake( cmd );
            cmd << Config::instance()->qmake_path();
            cmd << "make clean";
            cmd << "make";
            after_qmake( cmd );
            d_process->run( d_project.dir(), cmd );
         }
      }
   }
}
// end of build

//*******************************************************************
// process_break                                             PRIVATE
//*******************************************************************
void ProjectController::process_break()
{
   if( d_process ) {
      d_process->terminate();
      QTimer::singleShot( 1000, d_process, SLOT( kill() ) );
   }
}
// end of process_break

//*******************************************************************
// process_broken                                            PRIVATE
//********************************************************************
void ProjectController::process_broken()
{
   if( d_process ) delete d_process;
   d_process = 0;
}
// end of process_broken

//###################################################################
//#                                                                 #
//#                           D E B U G                             #
//#                                                                 #
//###################################################################

//*******************************************************************
// dbg_run                                                   PRIVATE
//*******************************************************************
void ProjectController::dbg_run()
{
   if( 0 == d_debugger ) {
      d_debugger = new Debugger();
      if( d_debugger ) {
         const QString program = d_project.prg_fpath();
         const QString wrkdir  = d_project.dir();
         d_debugger->start( program, wrkdir );
      }
   }
}

//*******************************************************************
// dbg_stop                                                  PRIVATE
//*******************************************************************
void ProjectController::dbg_stop()
{
   if( d_debugger ) {
      d_debugger->stop();
   }
}
// end of dbg_stop
