/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectFilesWidget.cpp
 * DATE   : 14.12.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectFilesWidget.h"
#include "Shared.h"
#include <QLayout>
#include <QListWidget>
#include <QPushButton>
#include <QFileDialog>
#include <QDir>
#include <QKeyEvent>
#include <QFile>
#include <QMessageBox>
#include <QScrollBar>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ProjectFilesWidget::MSG_MULTI      = QT_TR_NOOP( "Some of files have an invalid extension" );
const char* const ProjectFilesWidget::MSG_SINGLE     = QT_TR_NOOP( "The file %1 has an invalid extension" );
const char* const ProjectFilesWidget::CAPTION_CREATE = QT_TR_NOOP( "Project file creation" );
const char* const ProjectFilesWidget::CANT_CREATE    = QT_TR_NOOP( "Can't create the file: %1" );


//*******************************************************************
// ProjectFilesWidget                                    CONSTRUCTOR
//*******************************************************************
ProjectFilesWidget::ProjectFilesWidget( ProjectProp* const in_project_prop, QWidget* const in_parent )
: QWidget        ( in_parent )
, d_project_prop ( in_project_prop )
, d_list         ( new QListWidget )
, d_dir          ()
, d_filters      ()
{
   d_set_files.clear();
   d_out_files.clear();
   
   QHBoxLayout* const main_lay = new QHBoxLayout( this );
   d_list->setAlternatingRowColors( true );
   d_list->setSelectionMode( QListWidget::ExtendedSelection );
   main_lay->addWidget( d_list );
   QVBoxLayout* const btn_lay = new QVBoxLayout;
   QPushButton* const add_btn = new QPushButton( "&Add" );
   QPushButton* const del_btn = new QPushButton( "&Del" );
   btn_lay->addWidget( add_btn );
   btn_lay->addWidget( del_btn );
   btn_lay->addStretch();
   main_lay->addLayout( btn_lay );
   
   connect( add_btn, SIGNAL( clicked() ), this, SLOT( add_slot() ) );
   connect( del_btn, SIGNAL( clicked() ), this, SLOT( del_slot() ) );
}
// end of ProjectFilesWidget

//*******************************************************************
// keyPressEvent                                   PRIVATE inherited
//*******************************************************************
void ProjectFilesWidget::keyPressEvent( QKeyEvent* const in_event )
{
   QWidget::keyPressEvent( in_event );
   if( Qt::Key_Delete == in_event->key() ) {
      del_slot();
   }
}
// end of keyPressEvent

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void ProjectFilesWidget::showEvent( QShowEvent* const in_event )
{
   QWidget::showEvent( in_event );

   QScrollBar* const sb = d_list->horizontalScrollBar();
   if( sb ) {
      sb->setValue( sb->maximum() );
   }
}
// end of showEvent

//*******************************************************************
// add_file_to_list                                          PRIVATE
//*******************************************************************
void ProjectFilesWidget::add_file_to_list( const QString& in_fpath )
{
   // Plik nie moze byc wpisany na liste 2 razy.
   if( d_list->findItems( in_fpath, Qt::MatchFixedString | Qt::MatchCaseSensitive ).empty() ) {
      // Zakladamy, ze plik istnieje.
      bool add = true;

      // Jesli plik nie istnieje to tworzymy pusty plik
      // o podanej nazwie nazwie.
      const QFileInfo finfo( in_fpath );
      if( !finfo.exists() ) {
         add = Shared::create_empty_file( in_fpath );
      }

      // Dodajemy istniejacy plik.
      if( add ) {
         d_list->addItem( in_fpath );
      }
      else {
         QMessageBox::critical( 0, tr( CAPTION_CREATE ), tr( CANT_CREATE ).arg( in_fpath ) );
      }
   }
}
// end of add_file_to_list

//*******************************************************************
// add_slot                                             PRIVATE slot
//-------------------------------------------------------------------
// Dodanie plikow  do listy.
// Pliki wybiera sie z uzyciem dialogu. Mozna tez w nim wpisac nazwe
// nieistniejacego pliku. Akceptowane sa tylko pliki z odpowiednim
// rozszerzeniem. Jesli podany plik nie istnieje to zostanie utworzony.
//*******************************************************************
void ProjectFilesWidget::add_slot()
{
   if( d_dir.isEmpty() ) {
      d_dir = d_project_prop->dir();
      if( d_dir.isEmpty() ) {
         d_dir = QDir::homePath();
      }
   }

   QPointer< QFileDialog > dialog =  new QFileDialog( 0, QString(), d_dir );
   if( dialog ) {
      dialog->setFilters( d_filters );
      dialog->setAcceptMode( QFileDialog::AcceptOpen );
      dialog->setFileMode( QFileDialog::AnyFile );

      if( QDialog::Accepted == dialog->exec() ) {
         d_dir = dialog->directory().absolutePath();
         QStringList files = dialog->selectedFiles();
         if( !files.empty() ) {
            if( fname_is_valid( files ) ) {
               QStringList::const_iterator it = files.begin();
               const QStringList::const_iterator end = files.end();
               while( it != end ) {
                  add_file_to_list( *it );
                  ++it;
               }
               d_list->sortItems();   
            }
         }
      }
   }
}
// end of add_slot

//*******************************************************************
// del_slot                                             PRIVATE slot
//-------------------------------------------------------------------
// Usuniecie z listy aktualnie zaznaczonych pozycji.
//*******************************************************************
void ProjectFilesWidget::del_slot()
{
   const QList< QListWidgetItem* > items( d_list->selectedItems() );
   if( !items.empty() ) {
      QList< QListWidgetItem* >::const_iterator it = items.begin();
      const QList< QListWidgetItem* >::const_iterator end = items.end();
      while( it != end ) {
         delete d_list->takeItem( d_list->row( *it ) );
         ++it;
      }
   }
}
// end of del_slot

//*******************************************************************
// get_data                                                   PUBLIC
//-------------------------------------------------------------------
// Zwraca sciezki do wskazanych plikow.
//*******************************************************************
void ProjectFilesWidget::get_data( QStringList& out_files )
{
   out_files.clear();
   for( qint32 i = 0; i < d_list->count(); ++i ) {
      out_files << d_list->item( i )->text();
   }
   d_out_files = out_files;
}
// end of get_data

//*******************************************************************
// set_data                                                   PUBLIC
//*******************************************************************
void ProjectFilesWidget::set_data( const QStringList& in_files )
{
   QStringList::const_iterator it = in_files.begin();
   const QStringList::const_iterator end = in_files.end();

   d_list->clear();   
   while( it != end ) {
      add_file_to_list( *it );
      ++it;
   }

   d_set_files = in_files;
}
// end of set_data

//*******************************************************************
// changed                                                    PUBLIC
//*******************************************************************
bool ProjectFilesWidget::changed()
{
   if( d_out_files.empty() ) {
      get_data( d_out_files );
   }
   return ( d_out_files != d_set_files );
}
// end of changed

//*******************************************************************
// empty                                                      PUBLIC
//*******************************************************************
bool ProjectFilesWidget::empty() const
{
   return ( 0 == d_list->count() );
}
// end of empty

