/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Properties.cpp
 * DATE   : 26.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Properties.h"
#include "PropertiesGeneral.h"
#include "PropertiesFileTree.h"
#include "PropertiesClassTree.h"
#include "PropertiesEdit.h"
#include "PropertiesBuildTab.h"
#include "PropertiesMsgTab.h"
#include "PropertiesSearchTab.h"
#include "EventsController.h"
#include "Shared.h"
#include "Settings.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QShowEvent>
#include <QFontDialog>
#include <QListWidget>
#include <QStackedWidget>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const Properties::CAPTION              = QT_TR_NOOP( "Properties" );
const char* const Properties::ACCEPT               = QT_TR_NOOP( "&Accept" );
const char* const Properties::EXIT                 = QT_TR_NOOP( "E&xit" );
const char* const Properties::DEFAULT_FONT         = "Monospace,10,-1,5,50,0,0,0,0,0";
const char* const Properties::GROUP                = "Properties";
const char* const Properties::EDITOR_TEXT_FONT_KEY = "editor text font";
const char* const Properties::PROPTAB_NAME []      = {
	QT_TR_NOOP( "General" ),
	QT_TR_NOOP( "File Tree Tab" ),
	QT_TR_NOOP( "Class Tree Tab" ),
	QT_TR_NOOP( "Editor" ),
	QT_TR_NOOP( "Build Tab" ),
	QT_TR_NOOP( "Messages Tab" ),
	QT_TR_NOOP( "Search Tab" )
};

//*******************************************************************
// GotoLine                                              CONSTRUCTOR
//*******************************************************************
Properties::Properties( QWidget* const in_parent )
: QDialog( in_parent )
, d_main_layout ( new QHBoxLayout( this ) )
, d_wlist       ( new QListWidget( this ) )
, d_wstack      ( new QStackedWidget( this ) )
, d_btn_layout  ( new QVBoxLayout )
, d_prop_gen    ( new PropertiesGeneral( d_wstack ))
, d_prop_ftree  ( new PropertiesFileTree( d_wstack ) )
, d_prop_ctree  ( new PropertiesClassTree( d_wstack ) )
, d_prop_edit   ( new PropertiesEdit( d_wstack ) )
, d_prop_build  ( new PropertiesBuildTab( d_wstack ) )
, d_prop_msg    ( new PropertiesMsgTab( d_wstack ) )
, d_prop_search ( new PropertiesSearchTab( d_wstack ) )
, d_accept      ( new QPushButton( tr( ACCEPT ), this ) )
, d_exit        ( new QPushButton( tr( EXIT ), this ) )
{
	setWindowTitle( tr( CAPTION ) );
	
	const qint32 n = sizeof( PROPTAB_NAME ) / sizeof( PROPTAB_NAME[ 0 ] );
	qint32 maxlen = 0;
	for( qint32 i = 0; i < n; ++i ) {
		const QString txt( tr( PROPTAB_NAME[ i ] ) );
		d_wlist->insertItem( i, txt );
		if( txt.length() > maxlen ) {
			maxlen = txt.length();
		}
	}
	d_wlist->setMaximumWidth( ( maxlen + 2 ) * d_wlist->fontMetrics().width( "X" ) );
	d_wlist->setCurrentRow( 0 );

	d_wstack->insertWidget( 0, d_prop_gen    );
	d_wstack->insertWidget( 1, d_prop_ftree  );
	d_wstack->insertWidget( 2, d_prop_ctree  );
	d_wstack->insertWidget( 3, d_prop_edit   );
	d_wstack->insertWidget( 4, d_prop_build  );
	d_wstack->insertWidget( 5, d_prop_msg    );
	d_wstack->insertWidget( 6, d_prop_search );

	d_btn_layout->addStretch();
	d_btn_layout->addWidget( d_accept );
	d_btn_layout->addWidget( d_exit );

	d_main_layout->addWidget( d_wlist );
	d_main_layout->addWidget( d_wstack );
	d_main_layout->addLayout( d_btn_layout );
	
	connect( d_wlist , SIGNAL( currentRowChanged( int ) ), this, SLOT( row_changed( int ) ) );
	connect( d_accept, SIGNAL( clicked()  ), this, SLOT( accept() ) );
	connect( d_exit  , SIGNAL( clicked()  ), this, SLOT( reject() ) );
	connect( this    , SIGNAL( save_data() ), d_prop_gen    , SLOT( save_data() ) );
	connect( this    , SIGNAL( save_data() ), d_prop_ftree  , SLOT( save_data() ) ),
	connect( this    , SIGNAL( save_data() ), d_prop_ctree  , SLOT( save_data() ) ),
	connect( this    , SIGNAL( save_data() ), d_prop_edit   , SLOT( save_data() ) );
	connect( this    , SIGNAL( save_data() ), d_prop_build  , SLOT( save_data() ) );
	connect( this    , SIGNAL( save_data() ), d_prop_msg    , SLOT( save_data() ) );
	connect( this    , SIGNAL( save_data() ), d_prop_search , SLOT( save_data() ) );
}
// end of Properties

//*******************************************************************
// row_changed                                          PRIVATE slot
//*******************************************************************
void Properties::row_changed( const int in_row )
{
	d_wstack->setCurrentIndex( in_row );
}
// end of row_changed

//*******************************************************************
// accept                                               PRIVATE slot
//*******************************************************************
void Properties::accept()
{	
	emit save_data();
//	QDialog::accept();
}
// end of accept
