/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesWidget.cpp
 * DATE   : 30.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "PropertiesWidget.h"
#include <QLabel>
#include <QLayout>
#include <QCheckBox>
#include <QGroupBox>
#include <QLineEdit>
#include <QGridLayout>
#include <QPushButton>
#include <QFontDialog>
#include <QColorDialog>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const PropertiesWidget::USE_DEFAULTS  = QT_TR_NOOP( "Use &defaults" );
const char* const PropertiesWidget::USE_ALTBKG    = QT_TR_NOOP( "&Use alternate background color" );
const char* const PropertiesWidget::BACKGROUNDS   = QT_TR_NOOP( "Backgrounds" );
const char* const PropertiesWidget::NORMAL_BKG    = QT_TR_NOOP( "&Normal" );
const char* const PropertiesWidget::ALTERNATE_BKG = QT_TR_NOOP( "&Alternate" );
const char* const PropertiesWidget::FONT_TITLE    = QT_TR_NOOP( "Font" );
const char* const PropertiesWidget::FONT          = QT_TR_NOOP( "&Font" );
const char* const PropertiesWidget::FONT_COLOR    = QT_TR_NOOP( "&Color" );
const char* const PropertiesWidget::EXAMPLE_LBL   = QT_TR_NOOP( "Example text for presentation" );

//*******************************************************************
// PropertiesWidget                                      CONSTRUCTOR
//*******************************************************************
PropertiesWidget::PropertiesWidget( QWidget* const in_parent )
: QWidget          ( in_parent )
, d_mlay           ( new QVBoxLayout( this ) )
, d_defaults_cbx   ( new QCheckBox( tr( USE_DEFAULTS ), this ) )
, d_altbkg_cbx     ( new QCheckBox( tr( USE_ALTBKG ), this ) )
, d_gbox1          ( new QGroupBox( tr( BACKGROUNDS ), this ) )
, d_glay1          ( new QGridLayout( d_gbox1 ) )
, d_bkg_view       ( new QLineEdit( d_gbox1 ) )
, d_altbkg_view    ( new QLineEdit( d_gbox1 ) )
, d_bkg_btn        ( new QPushButton( QIcon( ":/img/colors.png" ), tr( NORMAL_BKG ), d_gbox1 ) )
, d_altbkg_btn     ( new QPushButton( QIcon( ":/img/colors.png" ), tr( ALTERNATE_BKG ), d_gbox1 ) )
, d_gbox2          ( new QGroupBox( tr( FONT_TITLE ), this ) )
, d_glay2          ( new QGridLayout( d_gbox2 ) )
, d_font_btn       ( new QPushButton( QIcon( ":/img/fonts.png" ), tr( FONT ), d_gbox2 ) )
, d_font_color_btn ( new QPushButton( QIcon( ":/img/colors.png" ), tr( FONT_COLOR ), d_gbox2 ) )
, d_example_lbl    ( new QLabel( tr( EXAMPLE_LBL ) ) )
{
	d_bkg_view->setMaximumHeight( 12 );
	d_bkg_view->setFocusPolicy( Qt::NoFocus );
	d_bkg_view->setReadOnly( true );

	d_altbkg_view->setMaximumHeight( 12 );
	d_altbkg_view->setFocusPolicy( Qt::NoFocus );
	d_altbkg_view->setReadOnly( true );

	d_glay1->addWidget( d_bkg_view   , 0, 0 );
	d_glay1->addWidget( d_altbkg_view, 0, 1 );
	d_glay1->addWidget( d_bkg_btn    , 1, 0 );
	d_glay1->addWidget( d_altbkg_btn , 1, 1 );

	d_glay2->addWidget( d_font_btn      , 0, 0 );
	d_glay2->addWidget( d_font_color_btn, 0, 1 );
	d_glay2->addWidget( d_example_lbl   , 1, 0, 1, 2, Qt::AlignHCenter );

	d_mlay->addWidget( d_defaults_cbx );
	d_mlay->addWidget( d_altbkg_cbx );
	d_mlay->addWidget( d_gbox1 );
	d_mlay->addWidget( d_gbox2 );
	d_mlay->addStretch( 1 );
	
	connect( d_defaults_cbx  , SIGNAL( clicked() ), this, SLOT( colors() ) );
	connect( d_bkg_btn       , SIGNAL( clicked() ), this, SLOT( select_bkg() ) );
	connect( d_altbkg_btn    , SIGNAL( clicked() ), this, SLOT( select_altbkg() ) );
	connect( d_font_btn      , SIGNAL( clicked() ), this, SLOT( select_font() ) );
	connect( d_font_color_btn, SIGNAL( clicked() ), this, SLOT( select_font_color() ) );
}
// end of PropertiesWidget

//*******************************************************************
// set_background                                            PRIVATE
//-------------------------------------------------------------------
// Ustawienie nowego koloru tla w polach sluzacych do demonstacji.
// Pola sa typu QLineEdit.
//*******************************************************************
void PropertiesWidget::set_background( QLineEdit* const in_led, const QRgb in_rgb )
{
	QPalette p;
	p.setColor( in_led->backgroundRole(), QColor::fromRgba( in_rgb ) );
	in_led->setPalette( p );
}
// end of set_background

//*******************************************************************
// set_example_lbl                                           PRIVATE
//-------------------------------------------------------------------
// Ustawienie fontu i koloru czcionki w napisie demostracyjnym.
//*******************************************************************
void PropertiesWidget::set_example_lbl( const QString& in_font_as_string, const QRgb in_rgb )
{
	QPalette p;
	p.setColor( d_example_lbl->foregroundRole(), QColor::fromRgba( in_rgb ) );
	d_example_lbl->setPalette( p );
	
	if( !d_font_as_string.isEmpty() ) {
		QFont font;
		font.fromString( in_font_as_string );
		d_example_lbl->setFont( font );

		QString txt = font.family();
		txt += ", ";
		txt += QString::number( font.pointSize() );
		txt += " pt.";
		d_example_lbl->setText( txt );
	}
}
// end of set_background

//*******************************************************************
// colors                                               PRIVATE slot
//*******************************************************************
void PropertiesWidget::colors()
{
	d_gbox1->setDisabled( d_defaults_cbx->isChecked() );
	d_gbox2->setDisabled( d_defaults_cbx->isChecked() );
}
// end of select_bkg

//*******************************************************************
// select_bkg                                           PRIVATE slot
//*******************************************************************
void PropertiesWidget::select_bkg()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( d_bkg_rgb, &ok, this );
	if( ok ) {
		set_background( d_bkg_view, d_bkg_rgb = qrgb );
	}
}
// end of select_bkg

//*******************************************************************
// select_altbkg                                        PRIVATE slot
//*******************************************************************
void PropertiesWidget::select_altbkg()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( d_altbkg_rgb, &ok, this );
	if( ok ) {
		set_background( d_altbkg_view, d_altbkg_rgb = qrgb );
	}
}
// end of select_altbkg

//*******************************************************************
// select_font                                          PRIVATE slot
//*******************************************************************
void PropertiesWidget::select_font()
{
	bool ok;
	const QFont font = QFontDialog::getFont( &ok, d_example_lbl->font(), this );
	if( ok ) {
		set_example_lbl( d_font_as_string = font.toString(), d_font_color );
	}
}
// end of select_font

//*******************************************************************
// select_font_color                                    PRIVATE slot
//*******************************************************************
void PropertiesWidget::select_font_color()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( d_font_color, &ok, this );
	if( ok ) {
		set_example_lbl( d_font_as_string, d_font_color = qrgb );
	}
}
// end of select_font_color
