/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtFind.cpp
 * DATE   : 23.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtFind.h"
#include "Config.h"
#include "Settings.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QPushButton>
#include <QStringList>
#include <QLineEdit>
#include <QLabel>
#include <QFontMetrics>
#include <QShowEvent>
#include <QList>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const QBtFind::FIND_CAPTION    = QT_TR_NOOP( "Find" );
const char* const QBtFind::REPLACE_CAPTION = QT_TR_NOOP( "Replace" );
const char* const QBtFind::SEARCH_FOR      = QT_TR_NOOP( "Search &for" );
const char* const QBtFind::REPLACE_WITH    = QT_TR_NOOP( "&Replace with" );
const char* const QBtFind::SEARCH_BACKWARD = QT_TR_NOOP( "search &backward" );
const char* const QBtFind::WHOLE_DOCUMENT  = QT_TR_NOOP( "in whole &document" );
const char* const QBtFind::SELECTED_TEXT   = QT_TR_NOOP( "in selected &text" );
const char* const QBtFind::WHOLE_WORDS     = QT_TR_NOOP( "&whole words" );
const char* const QBtFind::CASE_SENSITIVE  = QT_TR_NOOP( "case &sensitive" );
const char* const QBtFind::FIND            = QT_TR_NOOP( "&Find" );
const char* const QBtFind::CANCEL          = QT_TR_NOOP( "&Cancel" );


//*******************************************************************
// QBtFind                                                  CONSTRUCTOR
//*******************************************************************
QBtFind::QBtFind( QWidget* const in_parent, const QString& in_initial_text, const bool in_find )
: QDialog              ( in_parent )
, input_label_         ( new QLabel( tr( SEARCH_FOR ), this ) )
, input_               ( new QComboBox( this ) )
, replace_label_       ( new QLabel( tr( REPLACE_WITH ), this ) )
, replace_             ( new QComboBox( this ) )
, backward_cbx_        ( new QCheckBox( tr( SEARCH_BACKWARD ) ) )
, whole_document_cbx_  ( new QCheckBox( tr( WHOLE_DOCUMENT  ) ) )
, selected_text_cbx_   ( new QCheckBox( tr( SELECTED_TEXT   ) ) )
, whole_cbx_           ( new QCheckBox( tr( WHOLE_WORDS ), this ) )
, sensitive_cbx_       ( new QCheckBox( tr( CASE_SENSITIVE ), this ) )
, go_                  ( new QPushButton( tr( FIND ), this ) )
, cancel_              ( new QPushButton( tr( CANCEL ), this ) )
, initial_text_        ( in_initial_text.trimmed() )
, selected_text_active_( false )
{
   setWindowTitle( tr( in_find ? FIND_CAPTION : REPLACE_CAPTION ) );
   QFontMetrics fm( font() );

   input_label_->setBuddy( input_ );
   replace_label_->setBuddy( replace_ );
   
   input_->setEditable( true );
   input_->setMaxCount( MAX_ITEMS_NUMBER );
   input_->setMaxVisibleItems( MAX_ITEMS_NUMBER );
   input_->setDuplicatesEnabled( false );
   input_->setInsertPolicy( QComboBox::InsertAtTop );
   input_->setAutoCompletion( true );
   
   replace_->setEditable( true );
   replace_->setMaxCount( MAX_ITEMS_NUMBER );
   replace_->setMaxVisibleItems( MAX_ITEMS_NUMBER );
   replace_->setDuplicatesEnabled( false );
   replace_->setInsertPolicy( QComboBox::InsertAtTop );
   
    go_->setDefault( true );
    cancel_->setDefault( false );

   QGridLayout* const input_layout = new QGridLayout;
   input_layout->addWidget( input_label_, 0, 0 );
   input_layout->addWidget( input_, 0, 1 );
   input_layout->addWidget( replace_label_, 1, 0 );
   input_layout->addWidget( replace_, 1, 1 );
   input_layout->setColumnStretch( 1, 100 );
   input_layout->setColumnMinimumWidth( 1, 25 * fm.width( "X" ) );
   input_layout->setMargin( 4 );

   QVBoxLayout* const chk_layout = new QVBoxLayout;
   chk_layout->addWidget( backward_cbx_ );
   chk_layout->addWidget( whole_document_cbx_ );
   chk_layout->addWidget( selected_text_cbx_ );
   chk_layout->addWidget( whole_cbx_ );
   chk_layout->addWidget( sensitive_cbx_ );
   chk_layout->addStretch( 100 );

   QVBoxLayout* const btn_layout = new QVBoxLayout;
   btn_layout->addStretch( 100 );
   btn_layout->addWidget( cancel_ );
   btn_layout->addWidget( go_ );

   QHBoxLayout* const ctrl_layout = new QHBoxLayout;
   ctrl_layout->addLayout( chk_layout );
   ctrl_layout->addLayout( btn_layout );

   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addLayout( input_layout );
   main_layout->addLayout( ctrl_layout );
   setLayout( main_layout );

   connect( go_    , SIGNAL( clicked() ), this, SLOT( accept() ));
   connect( cancel_, SIGNAL( clicked() ), this, SLOT( reject() ));

   if( in_find ) {
      replace_label_->setVisible( false );
      replace_->setVisible( false );
      whole_document_cbx_->setVisible( false );
      selected_text_cbx_->setVisible( false );
   }
   else {
      backward_cbx_->setVisible( false );
   }
}
// end of QBtFind

//*******************************************************************
// ~QBtFind                                                  DESTRUCTOR
//*******************************************************************
QBtFind::~QBtFind()
{
   save_data();
}
// end of ~QBtFind

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtFind::showEvent( QShowEvent* const in_event )
{
   if( !selected_text_active_ ) {
      selected_text_cbx_->setEnabled( false );
   }
   QDialog::showEvent( in_event );
   restore_data();
   input_->setCurrentIndex( -1 );
   replace_->setCurrentIndex( -1 );
   if( !initial_text_.isEmpty() ) {
      input_->setEditText( initial_text_ );
   }
}
// end of showEvent

//*******************************************************************
// save_data                                                 PRIVATE
//*******************************************************************
void QBtFind::save_data()
{
   Settings stt;
   
   {   // QBtFind combo box
      QStringList slist;
      for( int i = 0; i < input_->count(); ++i ) {
         slist << input_->itemText( i );
      }
      stt.save( Config::GOTO_LINE_GROUP + Config::FIND_KEY, slist );
   }
   {   // replace combo box
      QStringList slist;
      for( int i = 0; i < replace_->count(); ++i ) {
         slist << replace_->itemText( i );
      }
      stt.save( Config::GOTO_LINE_GROUP + Config::REPLACE_KEY, slist );
   }
   {   // config
      QList<QVariant> cfg;
      cfg << backward_cbx_->isChecked();
      cfg << whole_cbx_->isChecked();
      cfg << sensitive_cbx_->isChecked();
      stt.save( Config::GOTO_LINE_GROUP + Config::CONFIG_KEY, cfg );
   }
}
// end of save_data

//*******************************************************************
// restore_data                                              PRIVATE
//*******************************************************************
void QBtFind::restore_data()
{
   Settings stt;
   QVariant data;
   
   // QBtFind combo box
   if( stt.read( Config::GOTO_LINE_GROUP + Config::FIND_KEY, data ) ) {
      input_->addItems( data.toStringList() );
   }
   // replace combo box
   if( stt.read( Config::GOTO_LINE_GROUP + Config::REPLACE_KEY, data ) ) {
      replace_->addItems( data.toStringList() );
   }
   // config
   if( stt.read( Config::GOTO_LINE_GROUP + Config::CONFIG_KEY, data ) ) {
      QList<QVariant> cfg = data.toList();
      if( 3 == cfg.size() ) {
         backward_cbx_->setChecked( cfg[ 0 ].toBool() );
         whole_cbx_->setChecked( cfg[ 1 ].toBool() );
         sensitive_cbx_->setChecked( cfg[ 2 ].toBool() );
      }
   }
}
// end of restore_data

//*******************************************************************
// accept                                               PRIVATE slot
//*******************************************************************
void QBtFind::accept()
{
   bool status = update_combo( input_ );

   if( replace_->isVisible() ) {
      status &= update_combo( replace_ );
   }

   if( status ) {
      QDialog::accept();
   }
}
// end of accept

//*******************************************************************
// update_combobox                                           PRIVATE
//*******************************************************************
bool QBtFind::update_combo( QComboBox* const in_box )
{
   bool retval = false;

   const QString text   = in_box->currentText();
   if( !text.isEmpty()  ) {
      if( -1 == in_box->findText( text ) ) {
         in_box->insertItem( 0, text );
      }
      retval = true;
   }
   return retval;
}
// end of update_combobox
